#!/usr/bin/env python3
import typing
from typing import List, NamedTuple

from conscommon.data_model import Device
from qtpy import QtGui, QtWidgets


class DeviceTreeSelection(NamedTuple):
    device: Device
    channels_selected: typing.List[bool]


class DeviceTreeView(QtWidgets.QWidget):
    def __init__(self):
        super(DeviceTreeView, self).__init__()

        self.tree = QtWidgets.QTreeView(self)
        layout = QtWidgets.QVBoxLayout(self)
        layout.addWidget(self.tree)

        self.model = QtGui.QStandardItemModel()
        self.model.dataChanged.connect(self.itemChanged)

        self.tree.header().setDefaultSectionSize(180)
        self.tree.setModel(self.model)
        self.model.itemChanged.connect(self.itemChanged)

        self.button = QtWidgets.QPushButton("Get")
        self.button.clicked.connect(self.getData)
        layout.addWidget(self.button)

        self.cont = True

    def importData(self, data: List[Device], root=None):

        self.model.clear()
        self.model.setHorizontalHeaderLabels(["Device", "Channel"])
        self.model.setRowCount(0)

        count = 0
        if root is None:
            root = self.model.invisibleRootItem()

        for d in data:
            item_dev = QtGui.QStandardItem(d.prefix)
            item_dev.setAutoTristate(False)
            item_dev.setCheckable(True)
            item_dev.setEditable(False)
            item_dev.setUserTristate(False)

            for ch in d.channels:
                i_ = QtGui.QStandardItem(ch.name)
                i_.setAutoTristate(False)
                i_.setCheckable(True)
                i_.setEditable(False)
                i_.setUserTristate(False)

                i_p = QtGui.QStandardItem(ch.prefix)
                i_p.setEditable(False)

                item_dev.appendRow([i_, i_p])

            root.appendRow(item_dev)
            count += 1
        self.model.setRowCount(count)

    def checkParentState(self, item) -> int:
        checked = 0
        for row in range(item.rowCount()):
            if item.child(row, 0).checkState() != 0:
                checked += 1

        if checked == item.rowCount():
            return 2
        elif checked == 0:
            return 0
        else:
            return 1

    def getData(self) -> List[DeviceTreeSelection]:
        root = self.model.invisibleRootItem()
        selected: List[DeviceTreeSelection] = []

        # Loop through devices
        for i in range(root.rowCount()):
            row = root.child(i, 0)

            if row.isCheckable() and row.checkState() != 0:
                channels = {}
                channels_selected = []
                for j in range(row.rowCount()):
                    channels_selected.append(row.child(j, 0).checkState() != 0)
                    channels[row.child(j, 0).text()] = {
                        "prefix": row.child(j, 1).text()
                    }

                selected.append(
                    DeviceTreeSelection(
                        Device(prefix=row.text(), channels=channels), channels_selected
                    )
                )
        return selected

    def paintItem(self, item, state):
        if state == 0:
            item.setBackground(QtGui.QColor("#fffff2"))
        elif state == 1:
            item.setBackground(QtGui.QColor("#ffffb2"))
        elif state == 2:
            item.setBackground(QtGui.QColor("#c0ffb2"))

    def iterChild(self, item):
        for row in range(item.rowCount()):
            yield item.child(row, 0), item.child(row, 1)

    def itemChanged(self, item):

        if not self.cont:
            return

        if type(item) == QtGui.QStandardItem:
            self.cont = False
            parent = item.parent()

            if item.rowCount() > 0 and parent is None:
                if item.checkState() == 0 or item.checkState() == 2:
                    for row in range(item.rowCount()):
                        if item.child(row, 0).checkState() != item.checkState():
                            item.child(row, 0).setCheckState(item.checkState())
            else:
                if parent and parent.isCheckable():
                    state = self.checkParentState(parent)
                    parent.setCheckState(state)

            _p = item if (parent is None and item.rowCount() > 0) else parent
            self.paintItem(_p, _p.checkState())
            for i1, i2 in self.iterChild(
                item if (parent is None and item.rowCount() > 0) else parent
            ):
                self.paintItem(i1, i1.checkState())
                self.paintItem(i2, i1.checkState())
            self.cont = True



import json
from functools import partial

import matplotlib.pyplot as plt
import numpy
from matplotlib.animation import FuncAnimation
from matplotlib.widgets import CheckButtons
from pydm import Display, PyDMApplication
from pydm.widgets import PyDMEmbeddedDisplay
from pydm.widgets.channel import PyDMChannel

from siriushlacon.countingpru.consts import LAYOUT_OVERVIEW_UI, OVERVIEW_UI

counters = ["C2", "C3", "M1"]  # Positions of conters on storage ring
Det_Location = ["M2", "C1", "C2", "C3", "C4"]  # Location of detectors


class Overview(Display):
    global counters, Det_Location

    def __init__(self, parent=None, macros=None, args=None, average="Gamma Detectors"):
        super().__init__(
            parent=parent, args=args, macros=macros, ui_filename=OVERVIEW_UI
        )
        self.setWindowTitle("Overview of Time Bases")
        self.alpha = [0, 0, 0, 0, 0]  # Initially doesn't show none graph
        self.average = average
        self.groups = ["{:0>2d}".format(sec) for sec in range(1, 21)]
        self.x = numpy.arange(len(self.groups))
        self.width = 0.185

        self.dict_pvs_tb = {}
        self.dict_macro_gamma = {}

        self.gamma_1 = [0] * 20
        self.gamma_2 = [0] * 20
        self.gamma_3 = [0] * 20
        self.gamma_4 = [0] * 20
        self.gamma_5 = [0] * 20

        self.fig, self.ax = plt.subplots(figsize=(12, 8))  #
        self.fig.canvas.set_window_title("Overview")  #
        self.fig.subplots_adjust(
            left=0.05, bottom=0.08, right=0.95, top=0.95
        )  # Adjustments of graphics
        plt.subplots_adjust(left=0.1)  #

        self.fig.text(0.03, 0.25, "Control of\n Graphic", ha="center")
        self.ani = FuncAnimation(fig=self.fig, func=self.animate, interval=10000)
        self.animate()
        self.checkButtons_setting()
        plt.show()

        if self.average == "Gamma Detectors":  # If user chose 'Counting - Overview'
            for PV in range(1, 21):
                for s_sec in range(2):
                    self.dict_pvs_tb[
                        "valueTB{}{}".format(PV, counters[s_sec])
                    ] = "ca://SI-{:0>2d}{}:CO-Counter:TimeBase-SP".format(
                        PV, counters[s_sec]
                    )
                if PV != 20:
                    self.dict_pvs_tb[
                        "valueTB{}M1".format(PV)
                    ] = "ca://SI-{:0>2d}M1:CO-Counter:TimeBase-SP".format(PV + 1)
                else:
                    self.dict_pvs_tb[
                        "valueTB{}M1".format(PV)
                    ] = "ca://SI-01M1:CO-Counter:TimeBase-SP"

            for location in range(1, 21):
                for s_sec in range(len(Det_Location)):
                    self.dict_macro_gamma[
                        "DET{}".format(s_sec)
                    ] = "SI-{:0>2d}{}:CO-Gamma".format(location, Det_Location[s_sec])
                    if s_sec < 3:
                        self.dict_macro_gamma["TimeBase{}".format(s_sec)] = "{}".format(
                            self.dict_pvs_tb[
                                "valueTB{}{}".format(location, counters[s_sec])
                            ]
                        )

                    a = PyDMChannel(
                        address="ca://SI-{:0>2d}{}:CO-Gamma:Count-Mon".format(
                            location, Det_Location[s_sec]
                        ),
                        value_slot=partial(self.plot, location=location, det=s_sec),
                    )  # Connect to Counting PVs
                    a.connect()

                self.disp = PyDMEmbeddedDisplay(
                    parent=self
                )  # Creates the window of Time Bases
                PyDMApplication.instance().close_widget_connections(self.disp)
                self.disp.macros = json.dumps(self.dict_macro_gamma)
                self.disp.filename = LAYOUT_OVERVIEW_UI
                self.disp.setMinimumWidth(300)
                self.disp.setMinimumHeight(140)
                self.verticalLayout.addWidget(self.disp)

                PyDMApplication.instance().establish_widget_connections(self.disp)
        else:  # If user chose some Average
            for location in range(1, 21):
                for s_sec in range(len(Det_Location)):
                    a = PyDMChannel(
                        address="ca://SI-{:0>2d}{}:CO-Gamma:{}-Mon".format(
                            location, Det_Location[s_sec], self.average
                        ),
                        value_slot=partial(self.plot, location=location, det=s_sec),
                    )  # Connect to Averages PVs
                    a.connect()

    def checkButtons_setting(self):  # Configures of check button
        visibility = [line.patches[0].get_alpha() == 1 for line in self.graph]
        self.rax = plt.axes(position=[0.005, 0.08, 0.05, 0.15])
        self.labels = ["     " + str(line.get_label()) for line in self.graph]
        self.check = CheckButtons(self.rax, self.labels, visibility)
        self.check.on_clicked(self.hide_show)

    def hide_show(self, label):  # Set graph Visibilities
        index = self.labels.index(label)
        if self.graph[index].patches[index].get_alpha() == 1:
            for i in range(20):
                self.graph[index].patches[i].set_alpha(0)
            self.alpha[index] = self.graph[index].patches[index].get_alpha()

        else:
            for i in range(20):
                self.graph[index].patches[i].set_alpha(1)
                self.alpha[index] = self.graph[index].patches[i].get_alpha()
        plt.draw()
        self.animate(i)

    def animate(self, *args):  # Function to update the graph
        self.ax.clear()
        self.rects1 = self.ax.bar(
            self.x - self.width * 2,
            self.gamma_1,
            self.width,
            label="M2",
            alpha=self.alpha[0],
        )
        self.rects2 = self.ax.bar(
            self.x - self.width,
            self.gamma_2,
            self.width,
            label="C1",
            alpha=self.alpha[1],
        )
        self.rects3 = self.ax.bar(
            self.x, self.gamma_3, self.width, label="C2", alpha=self.alpha[2]
        )
        self.rects4 = self.ax.bar(
            self.x + self.width,
            self.gamma_4,
            self.width,
            label="C3",
            alpha=self.alpha[3],
        )
        self.rects5 = self.ax.bar(
            self.x + self.width * 2,
            self.gamma_5,
            self.width,
            label="C4",
            alpha=self.alpha[4],
        )

        self.ax.set_title("Overview of {}".format(self.average))
        self.ax.set_xlabel("Sectors of Storage Ring")
        self.ax.set_ylabel("Pulses per second")
        self.ax.set_xticklabels(self.groups)
        self.ax.set_xticks(self.x)
        self.ax.set_yscale("log")
        self.ax.legend()

        self.autolabel(self.rects1, self.alpha[0] == 1)
        self.autolabel(self.rects2, self.alpha[1] == 1)
        self.autolabel(self.rects3, self.alpha[2] == 1)
        self.autolabel(self.rects4, self.alpha[3] == 1)
        self.autolabel(self.rects5, self.alpha[4] == 1)

        self.graph = [self.rects1, self.rects2, self.rects3, self.rects4, self.rects5]

    def autolabel(self, rects, vis):  # Sets the visualization of counting above of bars
        for rect in rects:
            height = rect.get_height()
            self.ax.annotate(
                "{}".format(height),
                xy=(rect.get_x() + rect.get_width() / 2, height),
                xytext=(0, 3),  # 3 points vertical offset
                textcoords="offset points",
                ha="center",
                va="bottom",
                fontsize=8,
                rotation=90,
                visible=vis,
            )

    def plot(self, value="", location="", det=""):  # Updates the list of last counts
        eval("self.gamma_{}".format(det + 1)).insert(location - 1, round(value, 5))
        del eval("self.gamma_{}".format(det + 1))[location]



#!/usr/bin/env python3
import logging as _logging
import sys as _sys
import typing as _typing

from pydm import Display as _Display
from qtpy.QtWidgets import QWidget as _QWidget

from siriushlacon.utils.command_runner import CommandRunner
from siriushlacon.utils.dialog import BaseDialog as _BaseDialog
from siriushlacon.vbc.consts import CONFIRMATION_MESSAGE_UI
from siriushlacon.vbc.images import WARNING as _WARNING_PIXMAP
from siriushlacon.vbc.scripts import commute_valve, set_venting_valve_state

_logger = _logging.getLogger(__name__)


class ConfirmationMessage(_Display):
    def __init__(
        self,
        parent=None,
        args=None,
        macros=None,
        prefix: _typing.Optional[str] = None,
        relay_number: _typing.Optional[int] = None,
        state: _typing.Optional[bool] = None,
    ):
        super(ConfirmationMessage, self).__init__(
            parent=parent, args=args, macros=macros, ui_filename=CONFIRMATION_MESSAGE_UI
        )

        self.label_2.setPixmap(_WARNING_PIXMAP)

        if not prefix or not relay_number:
            if len(_sys.argv) < 6:
                raise ValueError("Missing arguments")

            self.prefix: str = _sys.argv[5]
            self.relay_number: int = int(_sys.argv[6])
        else:
            self.relay_number = relay_number
            self.prefix = prefix

        # updating VALVE name
        if self.relay_number == 1:
            self.VALVE.setText("Pre-vacuum Valve?")
        elif self.relay_number == 2:
            self.VALVE.setText("Gate Valve?")
        elif self.relay_number == 3:
            self.VALVE.setText("Valve 3?")
        elif self.relay_number == 4:
            self.VALVE.setText("Valve 4?")
        elif self.relay_number == 5:
            self.VALVE.setText("Venting Valve?")

        self.CommuteValveYesCommand = CommandRunner(
            command=lambda: commute_valve(
                prefix=self.prefix, valve=self.relay_number, confirmed=True
            ),
            name=f"CommuteValve?Relay={self.relay_number},confirmed={True}",
            close_when_finished=True,
            parent_widget=self.parent(),
        )
        self.CommuteValveNoCommand = CommandRunner(
            command=lambda: commute_valve(
                prefix=self.prefix, valve=self.relay_number, confirmed=False
            ),
            name=f"CommuteValve?Relay={self.relay_number},confirmed={False}",
            close_when_finished=True,
            parent_widget=self.parent(),
        )

        self.buttonBox.rejected.connect(
            lambda *_: self.CommuteValveNoCommand.execute_command()
        )
        self.buttonBox.accepted.connect(
            lambda *_: self.CommuteValveYesCommand.execute_command()
        )


class ConfirmationMessageDialog(_BaseDialog):
    def __init__(
        self,
        parent: _typing.Optional[_QWidget],
        prefix: str,
        relay_number: int,
        window_title: str = "VBC - Confirmation Message",
        macros: _typing.Dict[str, str] = None,
    ) -> None:
        super().__init__(parent, window_title=window_title)
        display = ConfirmationMessage(
            parent=self, prefix=prefix, relay_number=relay_number, macros=macros
        )
        self.set_display(display=display)


class VentingValveConfirmationMessage(_Display):
    def __init__(
        self,
        state: bool,
        prefix: str,
        parent=None,
        args=None,
        macros=None,
    ):
        super(VentingValveConfirmationMessage, self).__init__(
            parent=parent, args=args, macros=macros, ui_filename=CONFIRMATION_MESSAGE_UI
        )
        self.label_2.setPixmap(_WARNING_PIXMAP)
        self.prefix = prefix
        self.relay_number = 5
        self.state = state

        text = "(on)" if state else "(off)"
        self.VALVE.setText(f"Venting Valve? {text}")

        self.CommuteValveYesCommand = CommandRunner(
            command=lambda: set_venting_valve_state(prefix=self.prefix, state=state),
            name=f"SetVentingValve?prefix={self.prefix},state={self.state}",
            close_when_finished=True,
            parent_widget=self.parent(),
        )

        self.buttonBox.rejected.connect(self._close)
        self.buttonBox.accepted.connect(
            lambda *_: self.CommuteValveYesCommand.execute_command()
        )

    def _close(self):
        if self.parent() and isinstance(self.parent(), _QWidget):
            self.parent().close()
        else:
            _logger.warning(f"unable to close widget {self}")


class VentingValveConfirmationMessageDialog(_BaseDialog):
    def __init__(
        self,
        parent: _typing.Optional[_QWidget],
        prefix: str,
        state: bool,
        window_title: str = "VBC - Turbo Venting Valve Confirmation Message",
        macros: _typing.Dict[str, str] = None,
    ) -> None:
        super().__init__(parent, window_title=window_title)
        display = VentingValveConfirmationMessage(
            parent=self,
            prefix=prefix,
            macros=macros,
            state=state,
        )
        self.set_display(display=display)



#!/usr/bin/env python3
import json
import re

from pydm import Display
from pydm.utilities import IconFont
from pydm.widgets import PyDMLabel, PyDMRelatedDisplayButton
from qtpy.QtWidgets import QLabel

from siriushlacon.agilent4uhv.consts import (
    AGILENT_DEVICE_MAIN,
    AGILENT_MAIN_UI,
    lazy_devices,
)
from siriushlacon.utils.widgets import (
    TableDataController,
    TableDataRow,
    get_byte_indicator,
    get_label,
)

DEVICES = lazy_devices.get()
(
    ALARM,
    CURRENT,
    PRESSURE,
    VOLTAGE,
    FAN_TEMPERATURE,
    TEMPERATURE,
    DEVICE_NAME,
    CH_CONFIG,
    AUTOSTART,
) = range(9)
BOOSTER, RING, BTS, LTB = range(4)
CH_REG = re.compile(r":[C][0-9]")


class UHVDataController(TableDataController):
    def init_table(self):
        self.table.setRowCount(self.table_batch)
        self.table.setColumnCount(len(self.horizontalHeaderLabels))
        self.table.setHorizontalHeaderLabels(self.horizontalHeaderLabels)

        self.table.setColumnCount(len(self.horizontalHeaderLabels))
        self.table.setHorizontalHeaderLabels(self.horizontalHeaderLabels)

        for actual_row in range(self.table_batch):
            # Channel Name
            self.table.setCellWidget(actual_row, 0, QLabel(""))
            # Device Name
            self.table.setCellWidget(actual_row, 1, QLabel(""))
            # Fan Temperature
            self.table.setCellWidget(actual_row, 2, get_label(self.table, "", ""))
            # Autostart
            self.table.setCellWidget(actual_row, 3, get_label(self.table, "", ""))

            # Pressure
            self.table.setCellWidget(
                actual_row,
                4,
                get_label(self.table, "", "", PyDMLabel.DisplayFormat.Exponential),
            )
            # Device Unit
            self.table.setCellWidget(actual_row, 5, get_label(self.table, "", ""))

            # Voltage
            self.table.setCellWidget(actual_row, 6, get_label(self.table, "", ""))
            # Current
            self.table.setCellWidget(
                actual_row,
                7,
                get_label(self.table, "", "", PyDMLabel.DisplayFormat.Exponential),
            )
            # Temperature
            self.table.setCellWidget(
                actual_row,
                8,
                get_label(
                    self.table,
                    "",
                    "",
                ),
            )
            # Error Code Mon
            self.table.setCellWidget(
                actual_row, 9, get_byte_indicator(self.table, "", "")
            )

            rel = PyDMRelatedDisplayButton(self.table, AGILENT_DEVICE_MAIN)
            rel.filenames = [AGILENT_DEVICE_MAIN]
            rel.openInNewWindow = True

            # HV State
            self.table.setCellWidget(actual_row, 10, get_label(self.table, "", ""))
            # Power Max
            self.table.setCellWidget(actual_row, 11, get_label(self.table, "", ""))
            # V Target
            self.table.setCellWidget(actual_row, 12, get_label(self.table, "", ""))
            # I Protect
            self.table.setCellWidget(actual_row, 13, get_label(self.table, "", ""))
            # Setpoint
            self.table.setCellWidget(
                actual_row,
                14,
                get_label(self.table, "", "", PyDMLabel.DisplayFormat.Exponential),
            )

            # Details
            self.table.setCellWidget(actual_row, 15, rel)

    def showHideColumn(self, _type, HIDE):
        if _type == ALARM:
            self.table.setColumnHidden(9, HIDE)
        elif _type == CURRENT:
            self.table.setColumnHidden(7, HIDE)
        elif _type == FAN_TEMPERATURE:
            self.table.setColumnHidden(2, HIDE)
        elif _type == PRESSURE:
            self.table.setColumnHidden(4, HIDE)
            self.table.setColumnHidden(5, HIDE)
        elif _type == VOLTAGE:
            self.table.setColumnHidden(6, HIDE)
        elif _type == TEMPERATURE:
            self.table.setColumnHidden(8, HIDE)
        elif _type == DEVICE_NAME:
            self.table.setColumnHidden(1, HIDE)
        elif _type == AUTOSTART:
            self.table.setColumnHidden(3, HIDE)
        elif _type == CH_CONFIG:
            for index in range(10, 15):
                self.table.setColumnHidden(index, HIDE)

    def filter(self, pattern):
        for data in self.table_data:
            data.render = (
                pattern in data.device.prefix or pattern in data.channel.prefix
            )

        self.update_content.emit()

    def load_table_data(self):
        for device in self.devices:
            if not device.enable:
                continue
            for channel in device.channels:
                if not channel.enable or CH_REG.match(channel.prefix[-3:]):
                    continue
                self.table_data.append(TableDataRow(device, channel, True))
        self.update_content.emit()

    def update_table_content(self):
        total_rows = len(self.table_data) * 4

        # Maximum Allowed
        if self.batch_offset >= total_rows:
            return

        # Adding New Content
        actual_row = 0
        dataset_row = 0

        iterable = range(self.batch_offset, self.table_batch + self.batch_offset)
        self.table.setVerticalHeaderLabels([str(i) for i in iterable])
        for tableRow in self.table_data:

            # To render or not to render  ...
            if (
                tableRow.render
                and dataset_row >= self.batch_offset
                and actual_row != self.table_batch
            ):
                self.table.setRowHidden(actual_row, False)

                # Channel Access
                device_ca = "ca://" + tableRow.device.prefix
                channel_ca = "ca://" + tableRow.channel.prefix

                # Datails Button Macro
                macros = {}
                macros["DEVICE"] = tableRow.device.prefix
                for channel in tableRow.device.channels:
                    macros["PREFIX_{}".format(channel.name)] = channel.prefix

                # Channel Name
                self.table.cellWidget(actual_row, 0).setText(tableRow.channel.prefix)
                # Device Name
                self.table.cellWidget(actual_row, 1).setText(tableRow.device.prefix)
                # Fan Temperature
                self.connect_widget(actual_row, 2, device_ca + ":FanTemperature-Mon")
                # Autostart-RB
                self.connect_widget(actual_row, 3, device_ca + ":Autostart-RB")

                # Pressure
                self.connect_widget(actual_row, 4, channel_ca + ":Pressure-Mon")
                # Device Unit
                self.connect_widget(actual_row, 5, device_ca + ":Unit-RB")

                # Voltage
                self.connect_widget(actual_row, 6, channel_ca + ":Voltage-Mon")
                # Current
                self.connect_widget(actual_row, 7, channel_ca + ":Current-Mon")
                # Temperature
                self.connect_widget(actual_row, 8, channel_ca + ":HVTemperature-Mon")
                # LSB
                self.connect_widget(actual_row, 9, channel_ca + ":ErrorCode-Mon")

                # HVState-RB
                self.connect_widget(actual_row, 10, channel_ca + ":HVState-RB")
                # PowerMax-RB
                self.connect_widget(actual_row, 11, channel_ca + ":PowerMax-RB")
                # VoltageTarget-RB
                self.connect_widget(actual_row, 12, channel_ca + ":VoltageTarget-RB")
                # CurrentProtect-RB
                self.connect_widget(actual_row, 13, channel_ca + ":CurrentProtect-RB")
                # Setpoint-RB
                self.connect_widget(actual_row, 14, channel_ca + ":Setpoint-RB")

                # Details
                self.connect_widget(actual_row, 15, None, json.dumps(macros))

                actual_row += 1
            dataset_row += 1

            if actual_row != self.table_batch:
                for row in range(actual_row, self.table_batch):
                    self.table.setRowHidden(row, True)


class UHV(Display):
    def __init__(self, parent=None, args=None, macros=None):
        super(UHV, self).__init__(
            parent=parent, args=args, macros=macros, ui_filename=AGILENT_MAIN_UI
        )
        horizontal_header_labels = [
            "Channel Name",  # 0
            "Device Name",  # 1
            "Fan Temperature",  # 2
            "Autostart",  # 3
            "Pressure",  # 4
            "Unit",  # 5
            "Voltage",  # 6
            "Current",  # 7
            "Temperature",  # 8
            "Error Code Mon LSB",  # 9
            "HV State",  # 10
            "Power Max",  # 11
            "V Target",  # 12
            "I Protect",  # 13
            "Setpoint",  # 14
            "Details",
        ]
        table_batch = 0
        for device in DEVICES:
            if not device.enable:
                continue
            for channel in device.channels:
                if not channel.enable or CH_REG.match(channel.prefix[-3:]):
                    continue
                table_batch += 1
        self.tdc = UHVDataController(
            self.table,
            devices=DEVICES,
            table_batch=table_batch,
            horizontal_header_labels=horizontal_header_labels,
        )

        self.chAlarms.stateChanged.connect(
            lambda: self.showHideColumn(ALARM, self.chAlarms)
        )
        self.chCurrent.stateChanged.connect(
            lambda: self.showHideColumn(CURRENT, self.chCurrent)
        )
        self.chPressure.stateChanged.connect(
            lambda: self.showHideColumn(PRESSURE, self.chPressure)
        )
        self.chVoltage.stateChanged.connect(
            lambda: self.showHideColumn(VOLTAGE, self.chVoltage)
        )
        self.chTemperature.stateChanged.connect(
            lambda: self.showHideColumn(TEMPERATURE, self.chTemperature)
        )
        self.chFanTemperature.stateChanged.connect(
            lambda: self.showHideColumn(FAN_TEMPERATURE, self.chFanTemperature)
        )
        self.chDeviceName.stateChanged.connect(
            lambda: self.showHideColumn(DEVICE_NAME, self.chDeviceName)
        )
        self.chDeviceName.stateChanged.connect(
            lambda: self.showHideColumn(DEVICE_NAME, self.chDeviceName)
        )
        self.chChConfig.stateChanged.connect(
            lambda: self.showHideColumn(CH_CONFIG, self.chChConfig)
        )
        self.chAutostart.stateChanged.connect(
            lambda: self.showHideColumn(AUTOSTART, self.chAutostart)
        )

        self.tfFilter.editingFinished.connect(lambda: self.filter(self.tfFilter.text()))

        self.btnNavLeft.clicked.connect(lambda: self.update_navbar(False))
        self.btnNavLeft.setIcon(IconFont().icon("arrow-left"))
        self.btnNavRight.clicked.connect(lambda: self.update_navbar(True))
        self.btnNavRight.setIcon(IconFont().icon("arrow-right"))

        self.showHideColumn(ALARM, False)
        self.showHideColumn(CURRENT, False)
        self.showHideColumn(VOLTAGE, False)
        self.showHideColumn(TEMPERATURE, False)
        self.showHideColumn(FAN_TEMPERATURE, False)
        self.showHideColumn(CH_CONFIG, False)
        self.showHideColumn(CH_CONFIG, False)
        self.showHideColumn(AUTOSTART, False)

    def update_navbar(self, increase=True):
        self.tdc.changeBatch(increase)

    def filter(self, pattern):
        self.tdc.filter(pattern)

    def showHideColumn(self, _type, chk):
        if type(chk) == bool:
            HIDE = not chk
        else:
            HIDE = not chk.isChecked()
        self.tdc.showHideColumn(_type, HIDE)



import logging
import time

from siriushlacon.vbc.epics import ACP, BBB, ProcessRecovery, Turbovac

_logger = logging.getLogger(__name__)


class ProcessRecoveryAction:
    def __init__(self, prefix: str):
        if not prefix:
            raise ValueError(f"parameter prefix cannot be empty {prefix}")
        self.prefix = prefix
        self._tick = 0.5

        self.process_recovery = ProcessRecovery(prefix=self.prefix)
        self.acp = ACP(prefix=self.prefix)
        self.bbb = BBB(prefix=self.prefix)
        self.turbovac = Turbovac(prefix=self.prefix)

    def run(self):
        self._clear_status()
        self._stage_1()
        self._stage_2()
        self._stage_3()
        self._stage_4()
        self._stage_5()

    def _clear_status(self):
        """clear all status PVs"""
        _logger.info("clear_status")
        self.process_recovery.set_all_clear()

    def _stage_1(self):
        """Stage 1:"""
        _logger.info("stage1")
        self.process_recovery.status1_pv.value = 1
        # turn ACP15 pump ON and wait 30 s
        self.acp.on_off_pv.value = 1
        # set ACP15 speed to maximum (6000 rpm)
        self.acp.speed_rpm.value = 6000
        # wait until pump receives command to turn on
        while self.acp.on_off_pv.value == 0:
            time.sleep(self._tick)

        time.sleep(30)

    def _stage_2(self):
        """Stage 2:"""
        _logger.info("stage2")
        # open pre-vacuum valve
        self.bbb.pre_vacuum_valve_sw_pv.value = 1

        # update UI checkbox status
        # epics.caput(PRE_VACUUM_VALVE_UI, 1)

        # wait pre-vacuum valve receives value to open
        while self.bbb.pre_vacuum_valve_sw_pv.value == 0:
            time.sleep(self._tick)

        self.process_recovery.status2_pv.value = 1

    def _stage_3(self):
        """Stage 3:"""
        _logger.info("stage3")
        # turn TURBOVAC pump ON
        self.turbovac.pzd1_sp_tevl_pv.value = 1
        self.turbovac.pzd1_sp_zrvl_pv.value = 1

        # wait until pump receives command to turn on
        while (self.turbovac.pzd1_sp_zrvl_pv.value == 0) & (
            self.turbovac.pzd1_sp_tevl_pv.value == 0
        ):
            time.sleep(self._tick)

        self.process_recovery.status3_pv.value = 1

    def _stage_4(self):
        """Stage 4:"""
        _logger.info("stage4")
        # wait TURBOVAC pump reaches 1200 Hz
        self.turbovac.pzd2_sp_pv.value = 1200
        self.turbovac.pzd1_sp_sxvl_pv.value = 1

        while self.turbovac.pzd2_rb_pv.value < 1200:
            time.sleep(self._tick)

        self.turbovac.pzd1_sp_sxvl_pv.value = 0
        self.process_recovery.status4_pv.value = 1

    def _stage_5(self):
        """Stage 5:"""
        _logger.info("stage5")
        # open gate valve (VAT)
        self.bbb.gate_valve_sw_pv.value = 1

        # update UI checkbox status
        # epics.caput(GATE_VALVE_UI, 1)

        # ---------------------------------------
        # read gate valve (VAT) status to check if it is really open
        loop = True
        while loop:
            Lo = self.bbb.valve_open_pv.value
            Lg = self.bbb.valve_closed_pv.value
            if Lo & (not Lg):
                loop = False
            time.sleep(self._tick)

        self.process_recovery.status5_pv.value = 1

        # complement value of PV to launch "Process Finished" window
        self.process_recovery.toggle()


def process_recovery(prefix: str):
    """
    this script do all the procedures to recover from a pressurized system (after a
    power failure, for example) if the pressure is in the range (5*10^-2 ~ 1*10^-8).
    It is divided in 5 stages, described as follow:
        -stage 1: turn ACP15 on and wait 30 s
        -stage 2: open pre-vacuum valve
        -stage 3: turn TURBOVAC on
        -stage 4: wait TURBOVAC frequency reaches 1200 Hz
        -stage 5: open gate valve
    """
    action = ProcessRecoveryAction(prefix=prefix)
    action.run()



import json
import logging

from pydm import Display
from qtpy.QtGui import QPixmap

from siriushlacon.agilent4uhv.consts import (
    AGILENT_CHANNEL,
    AGILENT_DEVICE,
    AGILENT_DEVICE_MAIN_UI,
)
from siriushlacon.utils.consts import CNPEM_IMG, LNLS_IMG

logger = logging.getLogger()


class DeviceMain(Display):
    def __init__(self, parent=None, args=None, macros=None):
        super(DeviceMain, self).__init__(
            parent=parent, args=args, macros=macros, ui_filename=AGILENT_DEVICE_MAIN_UI
        )

        logger.info("DeviceMain macros {}".format(macros))
        self.btn_device.filenames = [AGILENT_DEVICE]
        self.btn_device.macros = json.dumps({"PREFIX": macros["DEVICE"]})
        self.btn_device.openInNewWindow = True

        self.btn_ch1.filenames = [AGILENT_CHANNEL]
        self.btn_ch1.macros = json.dumps({"PREFIX": macros["PREFIX_C1"]})
        self.btn_ch1.openInNewWindow = True

        self.btn_ch2.filenames = [AGILENT_CHANNEL]
        self.btn_ch2.macros = json.dumps({"PREFIX": macros["PREFIX_C2"]})
        self.btn_ch2.openInNewWindow = True

        self.btn_ch3.filenames = [AGILENT_CHANNEL]
        self.btn_ch3.macros = json.dumps({"PREFIX": macros["PREFIX_C3"]})
        self.btn_ch3.openInNewWindow = True

        self.btn_ch4.filenames = [AGILENT_CHANNEL]
        self.btn_ch4.macros = json.dumps({"PREFIX": macros["PREFIX_C4"]})
        self.btn_ch4.openInNewWindow = True

        self.label_cnpem.setPixmap(QPixmap(CNPEM_IMG))
        self.label_lnls.setPixmap(QPixmap(LNLS_IMG))



#!/usr/bin/env python3
import asyncio
import logging
import math
import sys
import time
from datetime import datetime, timedelta
from typing import List

import epics
from conscommon.data import TIMEFMT
from conscommon.data_model import Device
from qtpy.QtCore import QObject, QRunnable, Signal

from siriushlacon.agilent4uhv.tree import DeviceTreeSelection

logger = logging.getLogger()

EPICS_TOUT = 1
CMD_TOUT = 1.0
TIMER_BETWEEN_DEVICES = 0.5

FIXED, STEP, STEP_TO_FIXED = "fixed", "step", "step_to_fixed"


class AgilentAsync(QObject):
    timerStatus = Signal(dict)
    started = Signal()
    finished = Signal()

    def __init__(self, *args, **kwargs):
        super(AgilentAsync, self).__init__(*args, **kwargs)

    async def toFixed(
        self, device: Device, voltage: int, channels_selected: List[bool]
    ):
        self.timerStatus.emit({"device": device.prefix, "status": "to Fixed"})

        pv = device.prefix + ":Step-SP"
        actual_value = epics.caget(device.prefix + ":Step-RB", timeout=EPICS_TOUT)
        if actual_value is None:
            logger.critical(
                "Failed to get {} value, aborting operation.".format(
                    device.prefix + ":Step-RB"
                )
            )
            self.timerStatus.emit(
                {"device": device.prefix, "status": "failed to get readback"}
            )
            return

        val = actual_value
        for _, selected, shift in zip(
            device.channels, channels_selected, range(len(channels_selected))
        ):
            if selected:
                val &= ~(1 << shift)

        logger.info(
            "Channel {}  {:04b} -> {:04b} {}".format(
                pv, actual_value, val, channels_selected
            )
        )

        self.timerStatus.emit(
            {
                "device": device.prefix,
                "status": 'to Fixed "{:04b}"->"{:04b}"'.format(actual_value, val),
            }
        )

        if epics.caput(pv, val, timeout=EPICS_TOUT) == 1:
            await asyncio.sleep(CMD_TOUT)

            for ch, selected in zip(device.channels, channels_selected):
                if not selected:
                    continue
                pv, val = ch.prefix + ":VoltageTarget-SP", voltage

                self.timerStatus.emit(
                    {
                        "device": device.prefix,
                        "status": "{} Voltage -> {}V".format(pv, val),
                    }
                )

                logger.info("set {} {}".format(pv, val))
                epics.caput(pv, val, timeout=EPICS_TOUT)

                # @fixme: Two devices at the same serial network should have an actual delay!
                time.sleep(TIMER_BETWEEN_DEVICES)
                await asyncio.sleep(CMD_TOUT)

            self.timerStatus.emit({"device": device.prefix, "status": "Done"})
        else:
            self.timerStatus.emit({"device": device.prefix, "status": "Failed"})

    async def toStep(self, device: Device, channels_selected: List[bool]):
        self.timerStatus.emit({"device": device.prefix, "status": "to Step"})

        pv = device.prefix + ":Step-SP"
        actual_value = epics.caget(device.prefix + ":Step-RB", timeout=EPICS_TOUT)
        if actual_value is None:
            logger.critical(
                "Failed to get {} value, aborting operation.".format(
                    device.prefix + ":Step-RB"
                )
            )
            self.timerStatus.emit(
                {"device": device.prefix, "status": "failed to get readback"}
            )
            return

        val = actual_value
        n = 0
        for _, selected in zip(device.channels, channels_selected):
            if selected:
                val |= 1 << n
            n += 1

        self.timerStatus.emit(
            {
                "device": device.prefix,
                "status": "to Step {:04b} -> {:04b}".format(actual_value, val),
            }
        )
        logger.info("Channel {}  {:04b} -> {:04b}".format(pv, actual_value, val))
        if epics.caput(pv, val, timeout=EPICS_TOUT) == 1:
            self.timerStatus.emit({"device": device.prefix, "status": "Done"})
        else:
            self.timerStatus.emit({"device": device.prefix, "status": "Failed"})
        await asyncio.sleep(CMD_TOUT)

    async def doWait(self, device, _delay):
        t_ini = datetime.now()
        delay = timedelta(seconds=_delay)

        tick = math.ceil(_delay / 100)
        t_now = datetime.now()
        t_elapsed = t_now - t_ini
        while t_elapsed < delay:
            remaining = delay - t_elapsed
            logger.info('Time remaining {} for device "{}".'.format(remaining, device))
            self.timerStatus.emit({"device": device.prefix, "status": remaining})
            await asyncio.sleep(tick)

            t_now = datetime.now()
            t_elapsed = t_now - t_ini

        logger.info(
            'Running final function "{}" at {} for device "{}".'.format(
                self.toFixed.__name__, datetime.now().strftime(TIMEFMT), device
            )
        )

    async def toStepToFix(
        self,
        device: Device,
        _delay: float,
        channels_selected,
    ):
        """Run a function then another ..."""
        logger.info(
            'Running initial function "{}" for device "{}". Next method in {} seconds.'.format(
                self.toStep.__name__, device, _delay
            )
        )
        await self.toFixed(device, 7000, channels_selected=channels_selected)

        await self.doWait(device=device, _delay=_delay)

        await self.toFixed(device, 5000, channels_selected=channels_selected)

        await self.doWait(device=device, _delay=_delay)

        await self.toFixed(device, 3000, channels_selected=channels_selected)

    async def handle(
        self,
        mode,
        step_to_fixed_delay: float,
        voltage: int,
        devices_selection: List[DeviceTreeSelection],
    ):
        if sys.version_info >= (3, 7):
            from asyncio import create_task
        else:
            loop = asyncio.get_event_loop()
            create_task = loop.create_task

        tasks = []
        for sel in devices_selection:
            if mode == FIXED:
                tasks.append(
                    create_task(
                        self.toFixed(
                            sel.device,
                            voltage=voltage,
                            channels_selected=sel.channels_selected,
                        )
                    )
                )

            elif mode == STEP:
                tasks.append(
                    create_task(
                        self.toStep(sel.device, channels_selected=sel.channels_selected)
                    )
                )

            elif mode == STEP_TO_FIXED:
                tasks.append(
                    create_task(
                        self.toStepToFix(
                            _delay=step_to_fixed_delay,
                            device=sel.device,
                            channels_selected=sel.channels_selected,
                        )
                    )
                )
            time.sleep(TIMER_BETWEEN_DEVICES)

        await asyncio.gather(*tasks)

    def asyncStart(
        self,
        mode,
        step_to_fixed_delay,
        voltage,
        devices_selection,
    ):
        if sys.version_info >= (3, 7):
            from asyncio import run as asyncio_run
        else:
            loop = asyncio.new_event_loop()
            asyncio.set_event_loop(loop)
            asyncio_run = loop.run_until_complete

        asyncio_run(
            self.handle(
                mode=mode,
                step_to_fixed_delay=step_to_fixed_delay,
                voltage=voltage,
                devices_selection=devices_selection,
            )
        )

        if sys.version_info < (3, 7):
            loop.close()


class AgilentAsyncRunnable(QRunnable):
    def __init__(
        self,
        agilentAsync: AgilentAsync,
        devices_selection: List[DeviceTreeSelection],
        mode,
        step_to_fixed_delay: float,
        voltage: int,
    ):
        super(AgilentAsyncRunnable, self).__init__()
        self.agilentAsync = agilentAsync
        self.mode = mode
        self.step_to_fixed_delay = step_to_fixed_delay
        self.voltage = voltage
        self.devices_selection = devices_selection

    def run(self):
        self.agilentAsync.started.emit()
        try:
            self.agilentAsync.asyncStart(
                mode=self.mode,
                step_to_fixed_delay=self.step_to_fixed_delay,
                voltage=self.voltage,
                devices_selection=self.devices_selection,
            )
        except Exception:
            logger.exception("Unexpected Error")
        finally:
            self.agilentAsync.finished.emit()



#!/usr/bin/env python3
import logging

from pydm import Display

from siriushlacon.utils.command_runner import CommandRunner
from siriushlacon.utils.images import LNLS_PIXMAP
from siriushlacon.vbc.confirmation_message import VentingValveConfirmationMessageDialog
from siriushlacon.vbc.consts import SIMPLE_WINDOW_UI
from siriushlacon.vbc.scripts import commute_valve

logger = logging.getLogger(__name__)


class DeviceMenu(Display):
    def __init__(self, parent=None, args=None, macros=None):
        super(DeviceMenu, self).__init__(
            parent=parent, args=args, macros=macros, ui_filename=SIMPLE_WINDOW_UI
        )
        self.lnlsLabel.setPixmap(LNLS_PIXMAP)
        self.lnlsLabel.setFixedSize(LNLS_PIXMAP.size())

        self.prefix: str = macros["IOC"]

        self.CommuteValve1Command = CommandRunner(
            command=lambda: commute_valve(prefix=self.prefix, valve=1, confirmed=True),
            name="CommuteValve1_True",
        )

        self.CommuteValve2Command = CommandRunner(
            command=lambda: commute_valve(prefix=self.prefix, valve=2, confirmed=True),
            name="CommuteValve2_True",
        )

        self.ChkBoxPreVacValve.clicked.connect(
            lambda *_args, **_kwargs: self.CommuteValve1Command.execute_command()
        )
        self.ChkGateValve.clicked.connect(
            lambda *_args, **_kwargs: self.CommuteValve2Command.execute_command()
        )

        self.btn_on_turbo_vent_valve.clicked.connect(
            lambda *_args: self.show_confirmation_message(True)
        )
        self.btn_off_turbo_vent_valve.clicked.connect(
            lambda *_args: self.show_confirmation_message(False)
        )

    def show_confirmation_message(self, state: bool):
        dialog = VentingValveConfirmationMessageDialog(self, self.prefix, state)
        dialog.show()



import logging as _logging
import typing as _typing

import conscommon.data_model
from qtpy.QtWidgets import QApplication as _QApplication

_logger = _logging.getLogger(__name__)


class LazyDevices:
    """Helper class for lazy loading"""

    def __init__(self, api_get_method):
        self.api_get_method = api_get_method
        self._data: _typing.List[conscommon.data_model.Device] = None

    def get(self) -> _typing.List[conscommon.data_model.Device]:
        if self._data:
            return self._data

        self._data = conscommon.data_model.getDevicesFromBeagles(
            conscommon.data_model.getBeaglesFromList(self.api_get_method())
        )
        return self._data


def close_qt_application(*args, **kwargs):
    app = _QApplication.instance()
    if app:
        _logger.info(f"Closing qt application '{app}'")
        app.quit()



#!/usr/bin/env python3
import logging
import re
from typing import List

import conscommon.data_model
from pydm import Display
from pydm.widgets.drawing import PyDMDrawingRectangle
from pydm.widgets.label import PyDMLabel
from qtpy.QtCore import QRect, Qt
from qtpy.QtGui import QBrush, QColor, QFont
from qtpy.QtWidgets import QFrame, QLabel

from siriushlacon.agilent4uhv.consts import lazy_devices
from siriushlacon.utils.consts import BO, OVERVIEW_UI, SI, TB, TS
from siriushlacon.utils.widgets import FlowLayout

logger = logging.getLogger()
DEVICES = lazy_devices.get()


class Overview(Display):
    def __init__(self, parent=None, args=None, macros=None):
        super(Overview, self).__init__(parent=parent, args=args, macros=macros)
        self.ch_reg = re.compile(r":[C][0-9]")
        self.macros = macros
        self.pvs: List[dict] = []
        self.load_pvs()
        self.mainArea.setWidgetResizable(True)
        layout = FlowLayout(self.scrollAreaContent)
        for pv in self.pvs:
            layout.addWidget(self.get_gauge(None, macros=pv))

    def load_pv_dict(
        self,
        device: conscommon.data_model.Device,
        channel: conscommon.data_model.Channel,
    ):
        if (
            (self.ch_reg.match(channel.prefix[-3:]))
            or (self.macros.get("TYPE") == BO and not channel.prefix.startswith(BO))
            or (self.macros.get("TYPE") == SI and not channel.prefix.startswith(SI))
            or (self.macros.get("TYPE") == TS and not channel.prefix.startswith(TS))
            or (self.macros.get("TYPE") == TB and not channel.prefix.startswith(TB))
        ):
            logger.info("Ignored {}".format(channel.prefix))
            return None

        self.pvs.append(
            {
                "PV": channel.prefix + ":Pressure-Mon",
                "DISP": channel.prefix + ":Pressure-Mon",
                "ALARM": channel.prefix + ":Pressure-Mon.STAT",
                "SEC.": device.info.sector,
                "RACK": device.info.rack,
                "RS485": device.info.serial_id,
            }
        )

    def load_pvs(self):
        for device in DEVICES:
            if not device.enable:
                continue
            for channel in device.channels:
                self.load_pv_dict(device, channel)

    def get_gauge(self, parent, macros):
        aux = []
        for k, v in macros.items():
            aux.append("{}\t{}\n".format(k, v))
        tooltip = "".join(aux)

        width = 320
        height = 100

        frame = QFrame(parent)
        frame.setGeometry(QRect(10, 10, width, height))
        frame.setMinimumSize(width, height)
        frame.setFrameShape(QFrame.StyledPanel)
        frame.setFrameShadow(QFrame.Raised)
        frame.setObjectName("frame")

        brush = QBrush(QColor(180, 180, 180))
        brush.setStyle(Qt.NoBrush)

        alarmRec = PyDMDrawingRectangle(frame)
        alarmRec.channel = "ca://{}".format(macros.get("ALARM", None))
        alarmRec.setGeometry(QRect(0, 0, width, height))
        alarmRec.setToolTip(tooltip)
        alarmRec.setProperty("alarmSensitiveContent", True)
        alarmRec.setProperty("brush", brush)
        alarmRec.setObjectName("alarmRec")
        # alarmRec.setStyleSheet("margin:5px; border:3px solid rgb(0, 0, 0);")

        lblName = QLabel(frame)
        lblName.setGeometry(QRect(width * 0.05, 50, width - width * 0.05, 20))
        font = QFont()
        font.setPointSize(12)
        lblName.setFont(font)
        lblName.setAlignment(Qt.AlignCenter)
        lblName.setText("{}".format(macros.get("DISP", None)))
        lblName.setObjectName("lblName")
        lblName.setToolTip(tooltip)

        lblVal = PyDMLabel(frame)
        lblVal.setGeometry(QRect(width * 0.05, 10, width - width * 0.05, 30))
        font = QFont()
        font.setPointSize(18)
        lblVal.setFont(font)
        lblVal.setToolTip(tooltip)
        lblVal.setAlignment(Qt.AlignCenter)
        lblVal.setProperty("showUnits", False)
        lblVal.setObjectName("lblVal")
        lblVal.channel = "ca://{}".format(macros.get("PV", None))
        lblVal.precisionFromPV = False
        lblVal.precision = 2
        if self.macros.get("FORMAT", "") == "EXP":
            lblVal.displayFormat = PyDMLabel.DisplayFormat.Exponential
        return frame

    def ui_filename(self):
        return OVERVIEW_UI

    def ui_filepath(self):
        return OVERVIEW_UI



#!/usr/bin/env python3

import json
import logging
import subprocess
import typing

from pydm import Display
from qtpy.QtGui import QPixmap

from siriushlacon.utils.command_runner import CommandRunner
from siriushlacon.utils.images import CNPEM_PIXMAP, LNLS_PIXMAP
from siriushlacon.vbc.consts import (
    ADVANCED_WINDOW_PY,
    PLAY_IMG,
    SIMPLE_WINDOW_PY,
    STOP_IMG,
    SYSTEM_WINDOW_UI,
    Finished,
)
from siriushlacon.vbc.OK_message import OkMessageDialog
from siriushlacon.vbc.scripts import check_pressure, process_off, process_on
from siriushlacon.vbc.warning_message import (
    VBCWarningMessageDialog as _VBCWarningMessageDialog,
)

logger = logging.getLogger(__name__)


class DeviceMenu(Display):
    def __init__(self, parent=None, args=None, macros=None):
        super(DeviceMenu, self).__init__(
            parent=parent, args=args, macros=macros, ui_filename=SYSTEM_WINDOW_UI
        )
        self.lblOn.setPixmap(QPixmap(PLAY_IMG))
        self.lblOff.setPixmap(QPixmap(STOP_IMG))

        self.lnlsLabel.setPixmap(LNLS_PIXMAP)
        self.lnlsLabel.setFixedSize(LNLS_PIXMAP.size())

        self.cnpemLabel.setPixmap(CNPEM_PIXMAP)
        self.cnpemLabel.setFixedSize(CNPEM_PIXMAP.size())

        self.WarningMessagePopen: typing.Optional[subprocess.Popen] = None

        self.prefix = macros["IOC"]

        # Command Runners
        self.ProcessOnCommand = CommandRunner(
            command=lambda: process_on(self.prefix), name="ProcessOn"
        )
        self.CheckPressureCommand = CommandRunner(
            command=lambda: check_pressure(prefix=self.prefix, first_time=False)
        )
        self.ProcessOffCommand = CommandRunner(
            command=lambda: process_off(prefix=self.prefix)
        )

        # Trigger (usually by an external script)
        self.Shell_PV_trigger_PRESSURIZED.toggled.connect(self.display_warning_message)
        self.Shell_PV_trigger_ON.toggled.connect(
            lambda *_args, **_kwargs: self.ProcessOnCommand.execute_command()
        )
        self.Shell_PV_Trigger_OK_MESSAGE_ON.toggled.connect(self._display_on_message)
        self.Shell_PV_Trigger_OK_MESSAGE_OFF.toggled.connect(self._display_off_message)

        self.btnSimpleTab.macros = json.dumps({"IOC": self.prefix})
        self.btnSimpleTab.filenames = [SIMPLE_WINDOW_PY]

        self.btnAdvancedTab.macros = json.dumps({"IOC": self.prefix})
        self.btnAdvancedTab.filenames = [ADVANCED_WINDOW_PY]

        # ON Button
        self.Shell_ON_button.clicked.connect(
            lambda: self.CheckPressureCommand.execute_command()
        )

        # OFF Button
        self.Shell_OFF_button.clicked.connect(
            lambda: self.ProcessOffCommand.execute_command()
        )

    def _display_ok_message(self, finished: Finished, *_):
        dialog = OkMessageDialog(
            parent=self, finished=finished, prefix=self.prefix, macros=self.macros()
        )
        dialog.show()

    def _display_on_message(self, *_):
        self._display_ok_message(finished=Finished.ON)

    def _display_off_message(self, *_):
        self._display_ok_message(finished=Finished.OFF)

    def display_warning_message(self):
        dialog = _VBCWarningMessageDialog(
            parent=self, prefix=self.prefix, macros=self.macros()
        )
        dialog.show()



import logging

from epics import caput
from pydm import Display

from siriushlacon.agilent4uhv.consts import AGILENT_DEVICE_UI

logger = logging.getLogger()


class AgilentDevice(Display):
    def __init__(self, parent=None, args=None, macros=None):
        super(AgilentDevice, self).__init__(
            parent=parent, args=args, macros=macros, ui_filename=AGILENT_DEVICE_UI
        )
        self.macros = macros
        print(macros)
        self.pv_protect_sp = "ca://{}:Protect-SP".format(self.macros["PREFIX"])
        self.pv_step_sp = "ca://{}:Step-SP".format(self.macros["PREFIX"])

        self.btnWriteProtect.clicked.connect(self.write_protect)
        self.btnWriteStep.clicked.connect(self.write_step)

    def write_protect(self):
        data = 0
        if self.chProtect1.isChecked():
            data |= 1 << 0
        if self.chProtect2.isChecked():
            data |= 1 << 1
        if self.chProtect3.isChecked():
            data |= 1 << 2
        if self.chProtect4.isChecked():
            data |= 1 << 3
        pv = self.pv_protect_sp.replace("ca://", "")
        data = int(data)
        caput(pvname=pv, wait=False, timeout=0.5, value=data)
        logger.info("Write protect {0}({0:4b}) to {1}".format(data, pv))

    def write_step(self):
        data = 0
        if self.chStep1.isChecked():
            data |= 1 << 0
        if self.chStep2.isChecked():
            data |= 1 << 1
        if self.chStep3.isChecked():
            data |= 1 << 2
        if self.chStep4.isChecked():
            data |= 1 << 3
        data = int(data)
        pv = self.pv_step_sp.replace("ca://", "")
        caput(pvname=pv, wait=False, timeout=0.5, value=data)
        logger.info("Write step {0}({0:4b}) to {1}".format(data, pv))



#!/usr/bin/env python3


def get_device_macro(device, a, b, c, **kwargs):
    return {
        "DEVICE": device,
        "A": a,
        "B": b,
        "C": c,
        "G1": kwargs["G1"],
        "G2": kwargs["G2"],
        "G3": kwargs["G3"],
        "G4": kwargs["G4"],
        "G5": kwargs["G5"],
        "G6": kwargs["G6"],
    }


def get_macro(device, gauge, *args):
    macro = {
        "DEVICE": device,
        "r1": "",
        "r2": "",
        "r3": "",
        "r4": "",
        "GAUGE": gauge,
        "channel": "",
        "GA": "",
        "GB": "",
    }

    macro["GA"] = args[0]
    macro["GB"] = args[1]

    if gauge == "A":
        macro["r1"] = "1"
        macro["r2"] = "2"
        macro["r3"] = "3"
        macro["r4"] = "4"
        macro["channel"] = "1"

    elif gauge == "B":
        macro["r1"] = "5"
        macro["r2"] = "6"
        macro["r3"] = "7"
        macro["r4"] = "8"
        macro["channel"] = "3"

    elif gauge == "C":
        macro["r1"] = "9"
        macro["r2"] = "10"
        macro["r3"] = "11"
        macro["r4"] = "12"
        macro["channel"] = "5"

    return macro



#!/usr/bin/env python3

from pydm import Display

from siriushlacon.spixconv.consts import SPIXCONV_MAIN


class DeviceMenu(Display):
    def __init__(self, parent=None, args=None, macros=None):
        super(DeviceMenu, self).__init__(
            parent=parent, args=args, macros=macros, ui_filename=SPIXCONV_MAIN
        )

        # defining macros for PyDMShellCommand
        # self.***object_name***.macros = json.dumps({"pv_prefix":"TB-04:PU-InjSept"})
        # self.***object_name***.macros = json.dumps({"pv_prefix":"BO-01D:PU-InjKckr"})



#!/usr/bin/env python3
import json
import logging

from pydm import Display
from pydm.widgets.related_display_button import PyDMRelatedDisplayButton
from qtpy.QtGui import QPixmap
from qtpy.QtWidgets import QGridLayout

from siriushlacon.regatron.consts import COMPLETE_MAIN, DATA_JSON, REGATRON_UI
from siriushlacon.utils.consts import CNPEM_IMG, LNLS_IMG

logger = logging.getLogger()


def load_data():
    data = None
    with open(DATA_JSON, "rb") as f:
        data = json.load(f)
    return data


def get_overview_detail(data):
    name = data["P"]
    overview = PyDMRelatedDisplayButton(name)
    overview.macros = [json.dumps(data)]
    overview.filenames = [COMPLETE_MAIN]
    overview.openInNewWindow = True
    overview.showIcon = False
    return overview


class Launcher(Display):
    DIP = "DIP"
    QUA = "QUA"
    SEX = "SEX"

    def __init__(self, parent=None, macros=None, **kwargs):
        super().__init__(parent=parent, ui_filename=REGATRON_UI)
        self.logo_cnpem.setPixmap(QPixmap(CNPEM_IMG))
        self.logo_lnls.setPixmap(QPixmap(LNLS_IMG))

        self.data = load_data()
        self.layoutDipoles = QGridLayout()
        self.layoutQuadrupoles = QGridLayout()
        self.layoutSextupoles = QGridLayout()

        self.tabDipoles.setLayout(self.layoutDipoles)
        self.tabQuadrupoles.setLayout(self.layoutQuadrupoles)
        self.tabSextupoles.setLayout(self.layoutSextupoles)

        self.dipole = []
        self.quadrupole = []
        self.sextupole = []

        for e in self.data:
            if e["type"] == self.QUA:
                self.quadrupole.append(e)
            elif e["type"] == self.DIP:
                self.dipole.append(e)
            elif e["type"] == self.SEX:
                self.sextupole.append(e)

        self.render(self.layoutDipoles, self.dipole)
        self.render(self.layoutQuadrupoles, self.quadrupole)
        self.render(self.layoutSextupoles, self.sextupole)

    def render(self, layout, data):
        i = 0
        for name in data:
            overview = get_overview_detail(name)
            layout.addWidget(overview, i, 0)
            i += 1
        layout.setRowStretch(len(data), 10)



import logging
import re
import typing

import conscommon.data_model
from pydm.widgets import PyDMByteIndicator, PyDMLabel, channel
from qtpy.QtCore import QObject, QPoint, QRect, QSize, Qt, Signal
from qtpy.QtGui import QColor
from qtpy.QtWidgets import QLayout, QSizePolicy, QStyle

from siriushlacon.utils.consts import TABLE_ALARMS_QSS

logger = logging.getLogger()


def get_label(
    parent=None,
    content: str = "",
    tooltip: str = "",
    displayFormat: PyDMLabel.DisplayFormat = PyDMLabel.DisplayFormat.Default,
    precision: int = 2,
    showUnits: bool = True,
):
    lbl = PyDMLabel(parent=parent, init_channel=content)
    lbl.precisionFromPV = False
    lbl.precision = precision
    lbl.displayFormat = displayFormat
    lbl.showUnits = showUnits
    lbl.alarmSensitiveBorder = True
    lbl.alarmSensitiveContent = True
    lbl.setStyleSheet(TABLE_ALARMS_QSS)
    if precision:
        lbl.precision_changed(precision)
    return lbl


def get_byte_indicator(parent, content, tooltip, **kwargs):
    byte = PyDMByteIndicator(parent, content)
    byte.offColor = QColor(59, 0, 0)
    byte.onColor = QColor(255, 0, 0)
    byte.showLabels = False
    byte.orientation = Qt.Horizontal
    byte.numBits = 12
    return byte


class FlowLayout(QLayout):
    def __init__(self, parent=None, margin=-1, hspacing=-1, vspacing=-1):
        super(FlowLayout, self).__init__(parent)
        self._hspacing = hspacing
        self._vspacing = vspacing
        self._items = []
        self.setContentsMargins(margin, margin, margin, margin)

    def __del__(self):
        del self._items[:]

    def addItem(self, item):
        self._items.append(item)

    def horizontalSpacing(self):
        if self._hspacing >= 0:
            return self._hspacing
        else:
            return self.smartSpacing(QStyle.PM_LayoutHorizontalSpacing)

    def verticalSpacing(self):
        if self._vspacing >= 0:
            return self._vspacing
        else:
            return self.smartSpacing(QStyle.PM_LayoutVerticalSpacing)

    def count(self):
        return len(self._items)

    def itemAt(self, index):
        if 0 <= index < len(self._items):
            return self._items[index]

    def takeAt(self, index):
        if 0 <= index < len(self._items):
            return self._items.pop(index)

    def expandingDirections(self):
        return Qt.Orientations(0)

    def hasHeightForWidth(self):
        return True

    def heightForWidth(self, width):
        return self.doLayout(QRect(0, 0, width, 0), True)

    def setGeometry(self, rect):
        super(FlowLayout, self).setGeometry(rect)
        self.doLayout(rect, False)

    def sizeHint(self):
        return self.minimumSize()

    def minimumSize(self):
        size = QSize()
        for item in self._items:
            size = size.expandedTo(item.minimumSize())
        left, top, right, bottom = self.getContentsMargins()
        size += QSize(left + right, top + bottom)
        return size

    def doLayout(self, rect, testonly):
        left, top, right, bottom = self.getContentsMargins()
        effective = rect.adjusted(+left, +top, -right, -bottom)
        x = effective.x()
        y = effective.y()
        lineheight = 0
        for item in self._items:
            widget = item.widget()
            hspace = self.horizontalSpacing()
            if hspace == -1:
                hspace = widget.style().layoutSpacing(
                    QSizePolicy.PushButton, QSizePolicy.PushButton, Qt.Horizontal
                )
            vspace = self.verticalSpacing()
            if vspace == -1:
                vspace = widget.style().layoutSpacing(
                    QSizePolicy.PushButton, QSizePolicy.PushButton, Qt.Vertical
                )
            nextX = x + item.sizeHint().width() + hspace
            if nextX - hspace > effective.right() and lineheight > 0:
                x = effective.x()
                y = y + lineheight + vspace
                nextX = x + item.sizeHint().width() + hspace
                lineheight = 0
            if not testonly:
                item.setGeometry(QRect(QPoint(x, y), item.sizeHint()))
            x = nextX
            lineheight = max(lineheight, item.sizeHint().height())
        return y + lineheight - rect.y() + bottom

    def smartSpacing(self, pm):
        parent = self.parent()
        if parent is None:
            return -1
        elif parent.isWidgetType():
            return parent.style().pixelMetric(pm, None, parent)
        else:
            return parent.spacing()


class TableDataRow:
    def __init__(
        self,
        device: conscommon.data_model.Device,
        channel: conscommon.data_model.Channel,
        render: bool,
    ) -> None:
        self.device: conscommon.data_model.Device = device
        self.channel: conscommon.data_model.Channel = channel
        self.render: bool = render


class TableDataController(QObject):

    update_content = Signal()

    table_batch = 24
    filter_pattern = None

    def __init__(
        self,
        table,
        devices: typing.List[conscommon.data_model.Device] = None,
        table_batch=24,
        horizontal_header_labels=None,
        **kwargs
    ):
        super().__init__()
        self.devices: typing.List[conscommon.data_model.Device] = (
            devices if devices else []
        )
        self.table_data: typing.List[TableDataRow] = []
        self.table = table
        self.table_batch = table_batch
        self.horizontalHeaderLabels = horizontal_header_labels

        self.MaxValue = 50
        self.MinValue = 1

        self.batch_offset: int = 0
        self.total_rows: int = 0

        self.init_table()
        self.update_content.connect(self.update_table_content)
        self.update_content.connect(self.resize)

        self.load_table_data()

    def resize(self):
        if self.table:
            self.table.resizeColumnsToContents()

    def init_table(self):
        self.table.setRowCount(self.table_batch)
        self.table.setColumnCount(len(self.horizontalHeaderLabels))
        self.table.setHorizontalHeaderLabels(self.horizontalHeaderLabels)

        # TODO: Impplement ...
        raise NotImplementedError("Subclass must implement abstract method")

    def filter(self, pattern):
        # TODO: Impplement ...
        raise NotImplementedError("Subclass must implement abstract method")

    def load_table_data(self):
        # TODO: Impplement ...
        raise NotImplementedError("Subclass must implement abstract method")

    def update_table_content(self):
        # TODO: Impplement ...
        raise NotImplementedError("Subclass must implement abstract method")

    def disconnect_widget(self, row, col):
        widget = self.table.cellWidget(row, col)
        widget.channel = None

    def connect_widget(
        self, row, col, channel_name=None, macros=None, connect_color=None
    ):
        widget = self.table.cellWidget(row, col)
        if channel_name:
            widget.channel = channel_name
        if macros:
            widget.macros = macros
        if connect_color:
            cell_color = channel.PyDMChannel(
                address="ca://" + channel_name, value_slot=lambda: self.setColor(row)
            )
            cell_color.connect()

    def setColor(
        self, row, col=3
    ):  # Function to define cell color according to the value
        try:
            vmax = self.MaxValue
            vmin = self.MinValue
            range = vmax - vmin
            cell = self.table.cellWidget(row, col)

            if cell.text() != "":
                ActualValue = float(re.match(r"(\d+)\.(\d+)", cell.text()).group())

                if ActualValue < vmin:
                    ActualValue = vmin
                if (ActualValue - vmin) / range <= 0.5:
                    R = int((510 / range) * (ActualValue - vmin))
                    G = 255

                elif ActualValue < 200:
                    if ActualValue > vmax:
                        ActualValue = vmax
                    R = 255
                    G = int(-(510 / range) * (ActualValue - vmax))

                else:
                    return ()
                cell.setStyleSheet("background-color: rgb{}".format((R, G, 00)))
        except Exception:
            print("Problema com setColor")
            logger.exception('Problema com "setColor"')

    def changeBatch(self, increase):
        if increase:
            if self.batch_offset < len(self.table_data):
                self.batch_offset += self.table_batch
                self.update_content.emit()
        else:
            if self.batch_offset != 0:
                self.batch_offset -= self.table_batch
                if self.batch_offset < 0:
                    self.batch_offset = 0
                self.update_content.emit()



#!/usr/bin/env python3
import logging

from conscommon.data_model.mks import MKS_SENSOR_COLD_CATHODE
from pydm import Display
from pydm.widgets.drawing import PyDMDrawingRectangle
from pydm.widgets.label import PyDMLabel
from qtpy.QtCore import QRect, Qt
from qtpy.QtGui import QBrush, QColor, QFont
from qtpy.QtWidgets import QFrame, QLabel

from siriushlacon.mks937b.consts import lazy_devices
from siriushlacon.utils.consts import BO, OVERVIEW_UI, SI, TB, TS
from siriushlacon.utils.widgets import FlowLayout

DEVICES = lazy_devices.get()
logger = logging.getLogger()


class Overview(Display):
    def __init__(self, parent=None, args=None, macros=None):
        super(Overview, self).__init__(parent=parent, args=args, macros=macros)
        print(macros)

        self.macros = macros
        self.pvs = []
        self.load_pvs()
        self.mainArea.setWidgetResizable(True)
        layout = FlowLayout(self.scrollAreaContent)
        for pv in self.pvs:
            layout.addWidget(self.get_gauge(None, pv=pv))

    def load_pvs(self):
        macro_type = self.macros.get("TYPE", "")

        for device in DEVICES:
            if not device.enable:
                continue

            for channel in device.channels:
                if channel.info.sensor != MKS_SENSOR_COLD_CATHODE:
                    continue

                if macro_type not in [BO, SI, TB, TS]:
                    logger.warning('Invalid type "{}".'.format(macro_type))
                    continue
                if (
                    (macro_type == BO and not channel.prefix.startswith(BO))
                    or (macro_type == TB and not channel.prefix.startswith(TB))
                    or (macro_type == SI and not channel.prefix.startswith(SI))
                    or (macro_type == TS and not channel.prefix.startswith(TS))
                ):
                    continue
                print(channel.prefix)

                self.pvs.append(
                    {
                        "PV": channel.prefix + ":Pressure-Mon-s",
                        "DISP": channel.prefix + ":Pressure-Mon",
                        "ALARM": channel.prefix + ":Pressure-Mon.STAT",
                        "DEVICE": device.prefix,
                        "SEC.": device.info.sector,
                        "RACK": device.info.rack,
                        "RS485": device.info.serial_id,
                    }
                )

    def get_gauge(self, parent, pv):
        aux = []
        for k, v in pv.items():
            aux.append("{}\t{}\n".format(k, v))
        tooltip = "".join(aux)

        width = 320
        height = 100

        frame = QFrame(parent)
        frame.setGeometry(QRect(10, 10, width, height))
        frame.setMinimumSize(width, height)
        frame.setFrameShape(QFrame.StyledPanel)
        frame.setFrameShadow(QFrame.Raised)
        frame.setObjectName("frame")

        brush = QBrush(QColor(180, 180, 180))
        brush.setStyle(Qt.NoBrush)

        alarmRec = PyDMDrawingRectangle(frame)
        alarmRec.channel = "ca://{}".format(pv.get("ALARM", None))
        alarmRec.setGeometry(QRect(0, 0, width, height * 0.8))
        alarmRec.setToolTip(tooltip)
        alarmRec.setProperty("alarmSensitiveContent", True)
        alarmRec.setProperty("brush", brush)
        alarmRec.setObjectName("alarmRec")

        alarmRecComm = PyDMDrawingRectangle(frame)
        alarmRecComm.channel = "ca://{}".format(
            pv.get("DEVICE", None) + ":Pressure:Read"
        )
        alarmRecComm.setGeometry(QRect(0, height * 0.8, width, height * 0.2))
        alarmRecComm.setToolTip(
            "Connection Indicator: {}\t{}".format(
                "DEVICE", pv.get("DEVICE", None) + ":Pressure:Read"
            )
        )
        alarmRecComm.setProperty("alarmSensitiveContent", True)
        alarmRecComm.setProperty("brush", brush)
        alarmRecComm.setObjectName("alarmRecComm")
        alarmRecComm.setStyleSheet(
            """
            border:1px solid rgb(214, 214, 214);
        """
        )

        lblName = QLabel(frame)
        lblName.setGeometry(QRect(width * 0.05, 50, width - width * 0.05, 20))
        font = QFont()
        font.setPointSize(12)
        lblName.setFont(font)
        lblName.setAlignment(Qt.AlignCenter)
        lblName.setText("{}".format(pv.get("DISP", None)))
        lblName.setObjectName("lblName")
        lblName.setToolTip(tooltip)

        font = QFont()
        font.setPointSize(12)

        lblComm = QLabel(frame)
        lblComm.setGeometry(QRect(10, 80, 190, 20))
        lblComm.setFont(font)
        lblComm.setAlignment(Qt.AlignCenter)
        lblComm.setText("COMM Status")
        lblComm.setObjectName("lblComm")
        lblComm.setToolTip(
            "Communication status to device {}".format(pv.get("DEVICE", ""))
        )

        lblCommPv = PyDMLabel(frame)
        lblCommPv.setGeometry(QRect(150, 80, 190, 20))
        lblCommPv.setFont(font)
        lblCommPv.setToolTip(
            "Communication status to device {}".format(pv.get("DEVICE", ""))
        )
        lblCommPv.setAlignment(Qt.AlignCenter)
        lblCommPv.setObjectName("lblCommPv")
        lblCommPv.channel = "ca://{}".format(
            pv.get("DEVICE", None) + ":Pressure:Read.STAT"
        )

        lblVal = PyDMLabel(frame)
        lblVal.setGeometry(QRect(width * 0.05, 10, width - width * 0.05, 30))
        font = QFont()
        font.setPointSize(18)
        lblVal.setFont(font)
        lblVal.setToolTip(tooltip)
        lblVal.setAlignment(Qt.AlignCenter)
        lblVal.setProperty("showUnits", False)
        lblVal.setObjectName("lblVal")
        lblVal.channel = "ca://{}".format(pv.get("PV", None))
        lblVal.precisionFromPV = False
        lblVal.precision = 2
        if self.macros.get("FORMAT", "") == "EXP":
            lblVal.displayFormat = PyDMLabel.DisplayFormat.Exponential
        return frame

    def ui_filename(self):
        return OVERVIEW_UI

    def ui_filepath(self):
        return OVERVIEW_UI



import logging

from bson.objectid import ObjectId
from PyQt5.QtGui import QIntValidator, QKeySequence
from PyQt5.QtWidgets import QShortcut
from qtpy import QtCore, QtWidgets, uic

from siriushlacon.epicstel.consts import (
    EPICSTEL_ITEMS_UI,
    EPICSTEL_LOGIN_UI,
    EPICSTEL_PV_UI,
    EPICSTEL_TEAM_UI,
    EPICSTEL_USER_UI,
)

logger = logging.getLogger()


class Login(QtWidgets.QDialog):
    logged_in = QtCore.Signal(tuple)

    def __init__(self, parent=None, macros=None, args=None):
        super(Login, self).__init__()
        uic.loadUi(EPICSTEL_LOGIN_UI, self)

        self.button_box.accepted.connect(self.handle_login)
        self.button_box.rejected.connect(self.destroy)
        self.show()

    def handle_login(self):
        self.logged_in.emit((self.username.text(), self.password.text()))


class AddUser(QtWidgets.QDialog):
    add_user = QtCore.Signal(tuple)

    def __init__(self, teams: list):
        super(AddUser, self).__init__()
        uic.loadUi(EPICSTEL_USER_UI, self)

        self.team_combobox.addItems(teams)
        self.id_box.setValidator(QIntValidator())
        self.add_user_btn_box.accepted.connect(self.handle_add_user)
        self.add_user_btn_box.rejected.connect(self.destroy)
        self.show()

    def handle_add_user(self):
        self.add_user.emit(
            (
                self.name_box.text(),
                int(self.id_box.text()),
                self.team_combobox.currentText(),
            )
        )


class AddPV(QtWidgets.QDialog):
    add_pv = QtCore.Signal(tuple)

    def __init__(self, groups: list):
        super(AddPV, self).__init__()
        uic.loadUi(EPICSTEL_PV_UI, self)

        self.group_combobox.addItems(groups)
        self.add_pv_btn_box.accepted.connect(self.handle_add_pv)
        self.add_pv_btn_box.rejected.connect(self.destroy)
        self.show()

    def handle_add_pv(self):
        self.add_pv.emit(
            (
                self.name_box.text(),
                float(self.min_box.text()),
                float(self.max_box.text()),
                float(self.timeout_box.text().split(" ")[0]),
                self.group_combobox.currentText(),
            )
        )


class AddTeam(QtWidgets.QDialog):
    add_team = QtCore.Signal(tuple)

    def __init__(self, users: list):
        super(AddTeam, self).__init__()
        uic.loadUi(EPICSTEL_TEAM_UI, self)
        self.users = users

        self.admin_combobox.addItems(users)
        self.users_combobox.addItems(users)
        self.add_team_btn_box.accepted.connect(self.handle_add_team)
        self.add_team_btn_box.rejected.connect(self.destroy)

        self.del_sc = QShortcut(QKeySequence("Del"), self)
        self.del_sc.activated.connect(self.del_user)

        self.add_btn.clicked.connect(self.add_user)
        self.show()

    def del_user(self):
        new_user_list = [
            self.users_list.item(i).text() for i in range(self.users_list.count())
        ]
        new_user_list.remove(self.users_list.currentItem().text())

        self.users_list.clear()
        self.users_list.addItems(new_user_list)

        self.users_combobox.clear()
        self.users_combobox.addItems(set(self.users) ^ set(new_user_list))

    def add_user(self):
        self.users_list.addItem(self.users_combobox.currentText())

        listed_users_set = set(
            [self.users_list.item(i).text() for i in range(self.users_list.count())]
        )

        self.users_combobox.clear()
        self.users_combobox.addItems(set(self.users) ^ listed_users_set)

    def handle_add_team(self):
        self.add_team.emit(
            (
                self.name_box.text(),
                self.admin_combobox.currentText(),
                [
                    self.users_list.item(i).text()
                    for i in range(self.users_list.count())
                ],
            )
        )


class EditItems(QtWidgets.QDialog):
    edit_items = QtCore.Signal(ObjectId, list, str, str)

    def __init__(
        self, id: ObjectId, prev_items: list, items: list, tab: str, field: str
    ):
        super(EditItems, self).__init__()
        uic.loadUi(EPICSTEL_ITEMS_UI, self)
        self.items, self.tab, self.field, self.id = items, tab, field, id

        self.items_list.addItems(prev_items)
        self.items_combobox.addItems(set(items) ^ set(prev_items))

        self.btn_box.accepted.connect(self.handle_edit_items)
        self.btn_box.rejected.connect(self.destroy)

        self.del_sc = QShortcut(QKeySequence("Del"), self)
        self.del_sc.activated.connect(self.del_item)
        self.add_btn.clicked.connect(self.add_item)
        self.show()

    def del_item(self):
        new_items_list = [
            self.items_list.item(i).text() for i in range(self.items_list.count())
        ]
        new_items_list.remove(self.items_list.currentItem().text())

        self.items_list.clear()
        self.items_list.addItems(new_items_list)

        self.items_combobox.clear()
        self.items_combobox.addItems(set(self.items) ^ set(new_items_list))

    def add_item(self):
        self.items_list.addItem(self.items_combobox.currentText())

        listed_users_set = set(
            [self.items_list.item(i).text() for i in range(self.items_list.count())]
        )

        self.items_combobox.clear()
        self.items_combobox.addItems(set(self.items) ^ listed_users_set)

    def handle_edit_items(self):
        self.edit_items.emit(
            self.id,
            [self.items_list.item(i).text() for i in range(self.items_list.count())],
            self.tab,
            self.field,
        )



#!/usr/bin/env python3
import os
import subprocess

from siriushlacon.mks937b.consts import MKS_OVERVIEW

os.environ["PYDM_DEFAULT_PROTOCOL"] = "ca://"

macros = '\'{"device": "MKS", "TYPE": "TS", "TITLE": "MKS 937b - TS"}\''
subprocess.Popen("pydm --hide-nav-bar -m " + macros + " " + MKS_OVERVIEW, shell=True)



#!/usr/bin/env python3
import argparse
import os

from siriushlacon.mks937b.consts import MKS_MAIN
from siriushlacon.utils.launch import LogLevel, launch_pydm

os.environ["PYDM_DEFAULT_PROTOCOL"] = "ca://"


if __name__ == "__main__":
    parser = argparse.ArgumentParser("VAC Pumps Controller")
    args = parser.parse_args()
    launch_pydm(displayfile=MKS_MAIN, hide_nav_bar=True, log_level=LogLevel.INFO)



import logging

from pydm import Display

from siriushlacon.agilent4uhv.consts import AGILENT_CHANNEL_UI

logger = logging.getLogger()


class Channel(Display):
    def __init__(self, parent=None, args=None, macros=None):
        super(Channel, self).__init__(
            parent=parent, args=args, macros=macros, ui_filename=AGILENT_CHANNEL_UI
        )

        logger.info("Channel macros {}".format(macros))



#!/usr/bin/env python3

import logging

from pydm import Display, PyDMApplication
from pydm.utilities import IconFont
from qtpy.QtWidgets import QLabel

from siriushlacon.mbtemp.consts import OVERVIEW_MAIN_UI, lazy_devices
from siriushlacon.utils.widgets import TableDataController, get_label

logger = logging.getLogger()
DEVICES = lazy_devices.get()


class MBTempTableDataController(TableDataController):
    def init_table(self):
        self.table.setRowCount(self.table_batch)
        self.table.setColumnCount(len(self.horizontalHeaderLabels))
        self.table.setHorizontalHeaderLabels(self.horizontalHeaderLabels)
        for actual_row in range(self.table_batch):
            # Channel Name
            self.table.setCellWidget(actual_row, 0, QLabel(""))
            # Device Name
            self.table.setCellWidget(actual_row, 1, QLabel(""))
            # Device Alpha
            self.table.setCellWidget(actual_row, 2, get_label(self.table, "", ""))
            # Temperature
            self.table.setCellWidget(actual_row, 3, get_label(self.table, "", ""))
            # Temperature Raw
            # self.table.setCellWidget(actual_row, 4, get_label(self.table, "", ""))

    def filter(self, pattern):
        if pattern != self.filter_pattern:
            self.filter_pattern = pattern if pattern is not None else ""
            self.batch_offset = 0
            self.filter_pattern = pattern

            for data in self.table_data:
                data[2] = (
                    self.filter_pattern in data[0] or self.filter_pattern in data[1]
                )
            self.update_content.emit()

    def load_table_data(self):
        self.table_data = []
        for device in self.devices:
            if not device.enable:
                continue
            for channel in device.channels:
                if not channel.enable:
                    continue
                self.table_data.append([channel.prefix, device.prefix, True])
        self.update_content.emit()

    def update_table_content(self):
        total_rows = len(self.table_data) * 4

        # Maximum Allowed
        if self.batch_offset >= total_rows:
            return

        # Adding New Content
        actual_row = 0
        dataset_row = 0
        self.table.setVerticalHeaderLabels(
            [
                str(i)
                for i in range(self.batch_offset, self.table_batch + self.batch_offset)
            ]
        )

        for channel, dev, render in self.table_data:
            if channel[-3:-1] == "CH" or actual_row == self.table_batch:
                continue

            # To render or not to render  ...
            if render and dataset_row >= self.batch_offset:
                self.table.setRowHidden(actual_row, False)
                # Channel Access
                device_ca = "ca://" + dev
                channel_ca = "ca://" + channel

                # Channel
                self.table.cellWidget(actual_row, 0).setText(channel)
                # Device
                self.table.cellWidget(actual_row, 1).setText(dev)
                # Alpha
                self.connect_widget(
                    actual_row, 2, device_ca + ":Alpha", connect_color=True
                )
                # Temp
                self.connect_widget(actual_row, 3, channel_ca, connect_color=True)
                # self.connect_widget(actual_row, 4, channel_ca + "-Raw")

                actual_row += 1

            dataset_row += 1

        for row in range(actual_row, self.table_batch):
            self.table.setRowHidden(row, True)

    def update_TempMaxMin(self, Maximum="", Minimum=""):
        if Maximum != "":
            self.MaxValue = Maximum
        if Minimum != "":
            self.MinValue = Minimum


class TableDisplay(Display):
    def __init__(self, parent=None, args=None, macros=None):
        super(TableDisplay, self).__init__(parent=parent, args=args, macros=macros)

        table_batch = 0
        for device in DEVICES:
            if not device.enable:
                continue
            for channel in device.channels:
                if not channel.enable:
                    continue
                table_batch += 1

        horizontal_header_labels = [
            "Channel Name",
            "Device Name",
            "Device Alpha",
            "Temperature",
        ]
        # "Temperature Raw",]

        self.tdc = MBTempTableDataController(
            self.table,
            devices=DEVICES,
            table_batch=table_batch,
            horizontal_header_labels=horizontal_header_labels,
        )

        self.tfFilter.editingFinished.connect(lambda: self.filter(self.tfFilter.text()))

        self.TempMax.valueChanged.connect(
            lambda: self.tdc.update_TempMaxMin(Maximum=self.TempMax.value())
        )

        self.TempMin.valueChanged.connect(
            lambda: self.tdc.update_TempMaxMin(Minimum=self.TempMin.value())
        )

        self.btnNavLeft.clicked.connect(lambda: self.update_navbar(False))
        self.btnNavLeft.setIcon(IconFont().icon("arrow-left"))
        self.btnNavRight.clicked.connect(lambda: self.update_navbar(True))
        self.btnNavRight.setIcon(IconFont().icon("arrow-right"))
        PyDMApplication.instance().hide_nav_bar = True

    def update_navbar(self, increase=True):
        self.tdc.changeBatch(increase)

    def filter(self, pattern):
        self.tdc.filter(pattern)

    def ui_filename(self):
        return OVERVIEW_MAIN_UI

    def ui_filepath(self):
        return OVERVIEW_MAIN_UI



#!/bin/bash
import datetime
import os
import platform

import pkg_resources
import pytz


def get_abs_path(relative):
    return pkg_resources.resource_filename(__name__, relative)


SP_TZ = pytz.timezone("America/Sao_Paulo")
TIME_ZERO = datetime.timedelta(0)
ARCHIVER_URL: str = "http://10.0.38.42/retrieval/data/getData.json"

IS_LINUX: bool = os.name == "posix" or platform.system() == "Linux"


TABLE_ALARMS_QSS: str = ""
with open(get_abs_path("css/table-alarm.qss")) as f:
    TABLE_ALARMS_QSS = "".join(f.readlines())

OVERVIEW_UI: str = get_abs_path("ui/overview.ui")

# Images
CNPEM_IMG: str = get_abs_path("images/CNPEM.jpg")
CNPEM_INVISIBLE_IMG: str = get_abs_path("images/CNPEM-invisible.png")
CNPEM_INVISIBLE_LOGO_IMG: str = get_abs_path("images/CNPEM-invisible-logo.png")
LNLS_IMG: str = get_abs_path("images/sirius-hla-as-cons-lnls.png")

LNLS_INVISIBLE_IMG: str = get_abs_path("images/lnls-invisible.png")
LTB_IMG: str = get_abs_path("images/ltb.png")
BOOSTER_IMG: str = get_abs_path("images/booster.png")
BTS_IMG: str = get_abs_path("images/btts.png")
STORAGE_RING_IMG: str = get_abs_path("images/storage_ring.png")
RINGB1A_IMG: str = get_abs_path("images/ringB1A.png")
RINGB2A_IMG: str = get_abs_path("images/ringB2A.png")

BO, SI, TB, TS = "BO", "SI", "TB", "TS"



import logging

from siriushlacon.utils.epics import create_connected_pv

_logger = logging.getLogger(__name__)


def check_pressure(prefix: str, first_time: bool):
    """
    this script runs after the user hits the "ON" button under the "system_tab.ui"
    window. It checks whether the pressure is lower or higher than 0.05 Torr.
    - if it's value is higher than 0.05, then the "process_on" script is executed
    - if it's value is lower, then the "system_pressurized" window will pop-up
    """
    _logger.info(f"check_pressure {prefix} first_time={first_time}")
    pressure_pv = create_connected_pv(pvname=f"{prefix}:BBB:Torr")
    torr_base_pv = create_connected_pv(pvname=f"{prefix}:BBB:TorrBase")
    torr_exp_pv = create_connected_pv(pvname=f"{prefix}:BBB:TorrExp")

    torr_base_msg_pv = create_connected_pv(pvname=f"{prefix}:BBB:TorrBaseMsg")
    torr_exp_msg_pv = create_connected_pv(pvname=f"{prefix}:BBB:TorrExpMsg")

    process_trigger_on_pv = create_connected_pv(pvname=f"{prefix}:Process:TriggerOn")
    process_trigger_pressurized_pv = create_connected_pv(
        pvname=f"{prefix}:Process:TriggerPressurized"
    )

    torr_base_msg_pv.value = torr_base_pv.value
    torr_exp_msg_pv.value = torr_exp_pv.value

    # update value showed in "system_pressurized.ui" window
    # if pressure value is bigger than 0.05 Torr, trigger "process_on" script
    if pressure_pv.value > 0.05:
        if not first_time:
            process_trigger_on_pv.value = 1
            process_trigger_on_pv.value = 0

    # if pressure is between 0.05 and 1*10**-8, trigger "process_recovery" script
    elif (pressure_pv.value < 0.05) & (pressure_pv.value > 10 ** -8):
        process_trigger_pressurized_pv.value = 1
        process_trigger_pressurized_pv.value = 0



#!/usr/bin/env python3
import sys
import typing as _typing

from pydm import Display
from qtpy.QtGui import QPixmap
from qtpy.QtWidgets import QWidget as _QWidget

from siriushlacon.utils.command_runner import CommandRunner
from siriushlacon.utils.dialog import BaseDialog as _BaseDialog
from siriushlacon.vbc.consts import CHECK_IMG, OK_MESSAGE_UI, Finished
from siriushlacon.vbc.scripts import clear_status_off, clear_status_on, clear_status_rec


class OkMessage(Display):
    def __init__(
        self,
        parent=None,
        args=None,
        macros=None,
        prefix: _typing.Optional[str] = None,
        finished: _typing.Optional[_typing.Union[Finished, str]] = None,
    ):
        super(OkMessage, self).__init__(
            parent=parent, args=args, macros=macros, ui_filename=OK_MESSAGE_UI
        )
        if not prefix or not finished:
            if len(sys.argv) < 6:
                raise ValueError(f"Invalid arguments {sys.argv}")

            prefix = sys.argv[5]
            finished = sys.argv[6]
            finished = finished.upper().replace(" ", "")
        if not prefix:
            raise ValueError(f"prefix '{prefix}' cannot be empty {self}")
        self.prefix: str = prefix

        self.label_2.setPixmap(QPixmap(CHECK_IMG))
        self.ClearStatusCommand: CommandRunner

        if finished == Finished.ON:
            self.ClearStatusCommand = CommandRunner(
                name=f"ClearStatusOn?prefix={self.prefix}",
                command=lambda: clear_status_on(prefix=self.prefix),
                close_when_finished=True,
                parent_widget=self.parent(),
            )
        elif finished == Finished.OFF:
            self.ClearStatusCommand = CommandRunner(
                name=f"ClearStatusOff?prefix={self.prefix}",
                command=lambda: clear_status_off(prefix=self.prefix),
                close_when_finished=True,
                parent_widget=self.parent(),
            )
        elif finished == Finished.REC:
            self.ClearStatusCommand = CommandRunner(
                name=f"ClearStatusRec?prefix={self.prefix}",
                command=lambda: clear_status_rec(prefix=self.prefix),
                close_when_finished=True,
                parent_widget=self.parent(),
            )
        else:
            raise ValueError(f"Invalid argument {finished}, required {Finished}")

        self.pushButton.clicked.connect(
            lambda *_args, **_kwargs: self.ClearStatusCommand.execute_command()
        )


class OkMessageDialog(_BaseDialog):
    def __init__(
        self,
        parent: _typing.Optional[_QWidget],
        prefix: str,
        finished: Finished,
        macros: _typing.Optional[_typing.Dict[str, str]],
        window_title: str = "VBC - Ok Message",
    ) -> None:
        super().__init__(parent, window_title=window_title, macros=macros)
        display = OkMessage(
            parent=self, macros=macros, finished=finished, prefix=prefix
        )
        self.set_display(display=display)



#!/usr/bin/env python3
import enum
import json
import logging
import re
import threading
from typing import List

import conscommon.data_model
from pydm import Display
from pydm.utilities import IconFont
from pydm.widgets import PyDMLabel, PyDMRelatedDisplayButton
from qtpy.QtCore import Qt
from qtpy.QtWidgets import QCheckBox, QLabel

from siriushlacon.mks937b.consts import DEVICE_MENU, MKS_MAIN_UI, lazy_devices
from siriushlacon.utils.widgets import TableDataController, TableDataRow, get_label

CH_REG = re.compile(r":[ABC][0-9]")
DEVICES = lazy_devices.get()

logger = logging.getLogger()


@enum.unique
class TableColumn(enum.Enum):
    Channel = "Channel"
    Device = "Device"
    Pressure = "Pressure"
    Alarm = "Alarm"
    Unit = "Unit"
    Protect = "Protect"
    Relay_1_SP = "Relay 1 SP"
    Relay_1_Hyst = "Relay 1 Hyst"
    Relay_5_SP = "Relay 5 SP"
    Relay_5_Hyst = "Relay 5 Hyst"
    Relay_7_SP = "Relay 7 SP"
    Relay_7_Hyst = "Relay 7 Hyst"
    Relay_11_SP = "Relay 11 SP"
    Relay_11_Hyst = "Relay 11 Hyst"
    Relay_12_SP = "Relay 12 SP"
    Relay_12_Hyst = "Relay 12 Hyst"
    Details = "Details"


class MKSTableDataController(TableDataController):
    def __init__(
        self,
        table,
        devices: List[conscommon.data_model.Device] = None,
        table_batch: int = 24,
        horizontal_header_labels: List[str] = None,
        **kwargs
    ):
        super().__init__(
            table,
            devices=devices,
            table_batch=table_batch,
            horizontal_header_labels=horizontal_header_labels,
            **kwargs
        )

    def init_table(self):
        self.table.setRowCount(self.table_batch)
        self.table.setColumnCount(len(self.horizontalHeaderLabels))
        self.table.setHorizontalHeaderLabels(self.horizontalHeaderLabels)

        for actual_row in range(self.table_batch):
            col = 0
            for col_name in TableColumn:
                if col_name in [TableColumn.Device, TableColumn.Channel]:
                    self.table.setCellWidget(actual_row, col, QLabel(""))

                elif col_name == TableColumn.Details:
                    rel = PyDMRelatedDisplayButton(self.table)
                    rel.filenames = [DEVICE_MENU]
                    rel.openInNewWindow = True
                    self.table.setCellWidget(actual_row, col, rel)
                    rel.show()
                else:
                    self.table.setCellWidget(
                        actual_row, col, get_label(self.table, showUnits=False)
                    )
                col += 1

    def filter(self, pattern):
        if pattern != self.filter_pattern:
            self.filter_pattern = pattern if pattern is not None else ""
            self.batch_offset = 0
            self.filter_pattern = pattern

            for tableDataRow in self.table_data:
                tableDataRow.render = (
                    self.filter_pattern in tableDataRow.device.prefix
                    or self.filter_pattern in tableDataRow.channel.prefix
                )
            self.update_content.emit()

    def load_table_data(self):
        for device in self.devices:
            if not device.enable:
                continue
            for channel in device.channels:
                if not channel.enable or CH_REG.match(channel.prefix[-3:]):
                    continue
                self.table_data.append(TableDataRow(device, channel, True))

        self.total_rows = self.table_data.__len__()
        self.update_content.emit()

    def generate_macros(self, dataRow: TableDataRow) -> str:
        macros = {}
        macros["DEVICE"] = dataRow.device.prefix
        for channel in dataRow.device.channels:
            macros["G{}".format(channel.num + 1)] = channel.prefix

            if channel.name == "A1":
                macros["A"] = channel.info.sensor
            if channel.name == "A1":
                macros["B"] = channel.info.sensor
            if channel.name == "C1":
                macros["C"] = channel.info.sensor

        return json.dumps(macros)

    def getRelayPV(self, col_name, channel: conscommon.data_model.Channel):
        if "Relay" not in col_name:
            return ""

        _data = col_name.split(" ")
        _relay_num = _data[1]
        _t = _data[2]
        sufix = "Hyst-RB" if _t == "Hyst" else "Setpoint-RB"
        if (
            (channel.num == 0 and _relay_num == "1")
            or (channel.num == 2 and _relay_num in ["5", "7"])
            or (channel.num in [4, 5] and _relay_num in ["11", "12"])
        ):
            return ":Relay{}:{}".format(_relay_num, sufix)
        else:
            return ""

    def update_table_row(self, actual_row, dataRow: TableDataRow):
        self.table.setRowHidden(actual_row, False)
        col = 0
        # Channel Access
        device_ca = "ca://" + dataRow.device.prefix
        channel_ca = "ca://" + dataRow.channel.prefix
        for tc in TableColumn:
            if tc == TableColumn.Device:
                self.table.cellWidget(actual_row, col).setText(dataRow.device.prefix)
            elif tc == TableColumn.Channel:
                self.table.cellWidget(actual_row, col).setText(dataRow.channel.prefix)
            elif tc == TableColumn.Pressure:
                self.connect_widget(actual_row, col, channel_ca + ":Pressure-Mon-s")
            elif tc == TableColumn.Alarm:
                self.connect_widget(actual_row, col, channel_ca + ":Pressure-Mon.STAT")
            elif tc == TableColumn.Unit:
                self.connect_widget(actual_row, col, device_ca + ":Unit")
            elif tc == TableColumn.Protect:
                self.connect_widget(
                    actual_row, col, channel_ca + ":ProtectionSetpoint-RB-s"
                )
            elif tc == TableColumn.Details:
                self.connect_widget(
                    actual_row, col, None, self.generate_macros(dataRow)
                )
            else:
                _pv = self.getRelayPV(tc.value, dataRow.channel)
                if _pv != "":
                    self.table.cellWidget(
                        actual_row, col
                    ).displayFormat = PyDMLabel.DisplayFormat.Exponential
                    self.connect_widget(actual_row, col, device_ca + _pv)
                else:
                    self.connect_widget(actual_row, col, _pv)
                    self.table.cellWidget(actual_row, col).displayFormat = 0
                    self.table.cellWidget(actual_row, col).value_changed("")

            col += 1

    def update_table_content(self):

        # Maximum Allowed
        if self.batch_offset >= self.total_rows:
            return

        # Adding New Content
        actual_row = 0
        dataset_row = 0
        self.table.setVerticalHeaderLabels(
            [
                str(i)
                for i in range(self.batch_offset, self.table_batch + self.batch_offset)
            ]
        )

        for d in self.table_data:
            if actual_row == self.table_batch:
                continue

            # To render or not to render  ...
            if d.render and dataset_row >= self.batch_offset:
                self.update_table_row(actual_row, d)
                actual_row += 1
            dataset_row += 1

        for row in range(actual_row, self.table_batch):
            self.table.setRowHidden(row, True)


class MKS(Display):
    def __init__(self, parent=None, args=None, macros=None):
        super(MKS, self).__init__(
            parent=parent, args=args, macros=macros, ui_filename=MKS_MAIN_UI
        )
        self.caput_lock = threading.RLock()
        self.caput_enable = True

        table_batch = 0
        for device in DEVICES:
            if not device.enable:
                continue
            for channel in device.channels:
                if not channel.enable or CH_REG.match(channel.prefix[-3:]):
                    continue
                table_batch += 1

        horizontal_header_labels = [tc.value for tc in TableColumn]
        self.tdc = MKSTableDataController(
            self.table,
            devices=DEVICES,
            table_batch=table_batch,
            horizontal_header_labels=horizontal_header_labels,
        )

        self.tfFilter.editingFinished.connect(lambda: self.filter(self.tfFilter.text()))

        self.btnNavLeft.clicked.connect(lambda: self.update_navbar(False))
        self.btnNavLeft.setIcon(IconFont().icon("arrow-left"))
        self.btnNavRight.clicked.connect(lambda: self.update_navbar(True))
        self.btnNavRight.setIcon(IconFont().icon("arrow-right"))

        self.checkBoxAlarm: QCheckBox
        self.checkBoxUnit: QCheckBox
        self.checkBoxProtect: QCheckBox
        self.checkBoxRelay1: QCheckBox
        self.checkBoxRelay5: QCheckBox
        self.checkBoxRelay7: QCheckBox
        self.checkBoxRelayPirani: QCheckBox

        self.checkBoxAlarm.stateChanged.connect(self.displayColumns)
        self.checkBoxUnit.stateChanged.connect(self.displayColumns)
        self.checkBoxProtect.stateChanged.connect(self.displayColumns)
        self.checkBoxRelay1.stateChanged.connect(self.displayColumns)
        self.checkBoxRelay5.stateChanged.connect(self.displayColumns)
        self.checkBoxRelay7.stateChanged.connect(self.displayColumns)
        self.checkBoxRelayPirani.stateChanged.connect(self.displayColumns)

        self.checkBoxAlarm.setCheckState(2)
        self.checkBoxUnit.setCheckState(2)
        self.checkBoxProtect.setCheckState(2)
        self.checkBoxRelay1.setCheckState(0)
        self.checkBoxRelay5.setCheckState(0)
        self.checkBoxRelay7.setCheckState(0)
        self.checkBoxRelayPirani.setCheckState(0)

        for i in range(6, 16):
            self.table.setColumnHidden(i, True)

    def displayColumns(self, *args, **kwargs):
        # fmt: off
        self.table.setColumnHidden(3, self.checkBoxAlarm.checkState() != Qt.Checked)
        self.table.setColumnHidden(4, self.checkBoxUnit.checkState() != Qt.Checked)
        self.table.setColumnHidden(5, self.checkBoxProtect.checkState() != Qt.Checked)

        self.table.setColumnHidden(6, self.checkBoxRelay1.checkState() != Qt.Checked)
        self.table.setColumnHidden(7, self.checkBoxRelay1.checkState() != Qt.Checked)

        self.table.setColumnHidden(8, self.checkBoxRelay5.checkState() != Qt.Checked)
        self.table.setColumnHidden(9, self.checkBoxRelay5.checkState() != Qt.Checked)
        self.table.setColumnHidden(10, self.checkBoxRelay7.checkState() != Qt.Checked)
        self.table.setColumnHidden(11, self.checkBoxRelay7.checkState() != Qt.Checked)

        self.table.setColumnHidden(12, self.checkBoxRelayPirani.checkState() != Qt.Checked)
        self.table.setColumnHidden(13, self.checkBoxRelayPirani.checkState() != Qt.Checked)
        self.table.setColumnHidden(14, self.checkBoxRelayPirani.checkState() != Qt.Checked)
        self.table.setColumnHidden(15, self.checkBoxRelayPirani.checkState() != Qt.Checked)
        # fmt: on

    def filter(self, pattern):
        self.tdc.filter(pattern)

    def update_navbar(self, increase=True):
        self.tdc.changeBatch(increase)



#!/usr/bin/env python3
import os
import subprocess

from siriushlacon.agilent4uhv.consts import AGILENT_OVERVIEW

os.environ["PYDM_DEFAULT_PROTOCOL"] = "ca://"
macros = (
    '\'{"device": "UHV", "TYPE": "TB", "FORMAT": "EXP", '
    ' "TITLE": "ION Pump Agilent 4UHV - TB"}\''
)

subprocess.Popen(
    "pydm --hide-nav-bar -m " + macros + " " + AGILENT_OVERVIEW, shell=True
)



from qtpy.QtGui import QIcon, QPixmap

from siriushlacon.utils.consts import CNPEM_IMG as _CNPEM_IMG
from siriushlacon.utils.consts import CNPEM_INVISIBLE_IMG as _CNPEM_INVISIBLE_IMG
from siriushlacon.utils.consts import (
    CNPEM_INVISIBLE_LOGO_IMG as _CNPEM_INVISIBLE_LOGO_IMG,
)
from siriushlacon.utils.consts import LNLS_IMG as _LNLS_IMG
from siriushlacon.utils.consts import LNLS_INVISIBLE_IMG as _LNLS_INVISIBLE_IMG

CNPEM_PIXMAP: QPixmap = QPixmap(_CNPEM_IMG)
CNPEM_ICON: QIcon = QIcon(CNPEM_PIXMAP)

CNPEM_INVISIBLE_PIXMAP: QPixmap = QPixmap(_CNPEM_INVISIBLE_IMG)
CNPEM_INVISIBLE_ICON: QIcon = QIcon(CNPEM_INVISIBLE_PIXMAP)

CNPEM_INVISIBLE_LOGO_PIXMAP: QPixmap = QPixmap(_CNPEM_INVISIBLE_LOGO_IMG)
CNPEM_INVISIBLE_LOGO_ICON: QIcon = QIcon(CNPEM_INVISIBLE_LOGO_PIXMAP)

LNLS_PIXMAP: QPixmap = QPixmap(_LNLS_IMG)
LNLS_ICON: QIcon = QIcon(LNLS_PIXMAP)

LNLS_INVISIBLE_PIXMAP: QPixmap = QPixmap(_LNLS_INVISIBLE_IMG)
LNLS_INVISIBLE_ICON: QIcon = QIcon(LNLS_INVISIBLE_PIXMAP)



import pkg_resources


def get_abs_path(filename):
    return pkg_resources.resource_filename(__name__, filename)


EPICSTEL_MAIN = get_abs_path("main.py")
EPICSTEL_MAIN_UI = get_abs_path("ui/main.ui")
EPICSTEL_LOGIN_UI = get_abs_path("ui/login.ui")
EPICSTEL_USER_UI = get_abs_path("ui/user.ui")
EPICSTEL_TEAM_UI = get_abs_path("ui/team.ui")
EPICSTEL_PV_UI = get_abs_path("ui/pv.ui")
EPICSTEL_ITEMS_UI = get_abs_path("ui/editlist.ui")

EPICSTEL_TABS = ["pvs", "users", "teams"]

EPICSTEL_HOST = "mongodb://10.0.38.46:27040/"



import logging
import time

from siriushlacon.utils.epics import create_connected_pv

_logger = logging.getLogger(__name__)


def set_venting_valve_state(prefix: str, state: bool):
    """Set venting valve state :param state: On=True, Off=False"""
    _logger.info(f"set turbo venting valve state {state}")
    turbovac_venting_valve_pv = create_connected_pv(
        pvname=f"{prefix}:TURBOVAC:VentingValve-SW"
    )

    turbovac_venting_valve_pv.value = 1 if state else 0


def _commute_venting_valve(prefix: str, confirmed: bool):
    """
    this is script is used to commute a valve value
    """
    _logger.info(f"{prefix} confirmed={confirmed}")
    turbovac_venting_valve_pv = create_connected_pv(
        pvname=f"{prefix}:TURBOVAC:VentingValve-SW"
    )
    turbovac_venting_valve_ui_pv = create_connected_pv(
        pvname=f"{prefix}:TURBOVAC:VentingValve-UI.RVAL"
    )

    time.sleep(1)  # why?!
    _logger.info(f"pv sw update {prefix} confirmed={confirmed}")

    # if relay swtiching message is confirmed, change PV SW values:
    if confirmed:
        turbovac_venting_valve_pv.value = not (turbovac_venting_valve_pv.value)

    # if relay swtiching message is canceled, do nothing:
    else:
        turbovac_venting_valve_ui_pv.value = turbovac_venting_valve_pv.value


def _relay(prefix: str, valve: int, confirmed: bool):
    """
    this is script is used to commute a valve value
    """
    _logger.info("relay  prefix={prefix} valve={valve} confirmed={confirmed}")

    sw_pv = create_connected_pv(pvname=f"{prefix}:BBB:Relay{valve}-SW")
    ui_pv = create_connected_pv(pvname=f"{prefix}:BBB:Relay{valve}-UI.RVAL")

    time.sleep(1)  # why?!

    _logger.info(f"pv sw update prefix={prefix} valve={valve} confirmed={confirmed}")
    # if relay swtiching message is confirmed, change PV SW values:
    if confirmed:
        sw_pv.value = not (sw_pv.value)

    # if relay swtiching message is canceled, do nothing:
    else:
        ui_pv.value = sw_pv.value


def commute_valve(prefix: str, valve: int, confirmed: bool):
    """
    this is script is used to commute a valve value
    """
    _logger.info(f"prefix={prefix} value={valve} confirmed={confirmed}")
    if valve >= 1 and valve <= 4:
        _relay(prefix=prefix, valve=valve, confirmed=confirmed)
    elif valve == 5:
        _commute_venting_valve(prefix=prefix, confirmed=confirmed)
    else:
        raise ValueError(f"Invalid valve value '{valve}'")



import copy
import enum
import logging
import multiprocessing
import sys
import typing

import pydm
import pydm.data_plugins


@enum.unique
class LogLevel(int, enum.Enum):
    DEBUG = logging.DEBUG
    INFO = logging.INFO
    WARNING = logging.WARNING
    ERROR = logging.ERROR
    CRITICAL = logging.CRITICAL


class PyDMApp(pydm.PyDMApplication):
    def __init__(
        self,
        ui_file=None,
        command_line_args=None,
        display_args=None,
        perfmon=False,
        hide_nav_bar=False,
        hide_menu_bar=False,
        hide_status_bar=False,
        read_only=False,
        macros=None,
        use_main_window=True,
        stylesheet_path=None,
        fullscreen=False,
    ):
        if not command_line_args:
            command_line_args = []
        if not display_args:
            display_args = []
        super().__init__(
            ui_file=ui_file,
            command_line_args=command_line_args,
            display_args=display_args,
            perfmon=perfmon,
            hide_nav_bar=hide_nav_bar,
            hide_menu_bar=hide_menu_bar,
            hide_status_bar=hide_status_bar,
            read_only=read_only,
            macros=macros,
            use_main_window=use_main_window,
            stylesheet_path=stylesheet_path,
            fullscreen=fullscreen,
        )

    def new_pydm_process(self, ui_file, macros=None, command_line_args=None):
        kwargs = copy.deepcopy(
            {
                "displayfile": ui_file,
                "macros": macros,
                "hide_nav_bar": self.hide_nav_bar,
                "hide_menu_bar": self.hide_menu_bar,
                "hide_status_bar": self.hide_status_bar,
                "fullscreen": self.fullscreen,
                "read_only": pydm.data_plugins.is_read_only(),
                "perfmon": self.perfmon,
                "log_level": LogLevel.INFO,
            }
        )

        process = multiprocessing.Process(
            target=launch_pydm,
            kwargs=kwargs,
            args=command_line_args if command_line_args else [],
            daemon=False,
        )
        process.start()


def launch_pydm(
    displayfile: str,
    macros: typing.Union[str, typing.Dict[str, str]] = None,
    hide_nav_bar: bool = True,
    hide_menu_bar: bool = False,
    hide_status_bar: bool = False,
    fullscreen: bool = False,
    read_only: bool = False,
    perfmon: bool = False,
    log_level: LogLevel = LogLevel.INFO,
    *display_args,
):
    _display_args = list(display_args)
    if not macros:
        macros = {}

    logger = logging.getLogger("")
    handler = logging.StreamHandler()
    formatter = logging.Formatter(
        "[%(asctime)s %(levelname)-8s %(filename)s:%(lineno)s - %(funcName)s] %(message)s"
    )
    handler.setFormatter(formatter)
    logger.addHandler(handler)
    logger.setLevel("INFO")
    handler.setLevel("INFO")

    from pydm.utilities import setup_renderer

    setup_renderer()

    try:
        """
        We must import QtWebEngineWidgets before creating a QApplication
        otherwise we get the following error if someone adds a WebView at Designer:
        ImportError: QtWebEngineWidgets must be imported before a QCoreApplication instance is created
        """
        from qtpy import QtWebEngineWidgets  # noqa: F401
    except ImportError:
        logger.debug("QtWebEngine is not supported.")

    import pydm
    from pydm.utilities.macro import parse_macro_string

    if macros is not None and type(macros) == str:
        macros = parse_macro_string(macros)

    logger.setLevel(log_level)
    handler.setLevel(log_level)

    app = PyDMApp(
        ui_file=displayfile,
        command_line_args=_display_args,
        perfmon=perfmon,
        hide_nav_bar=hide_nav_bar,
        hide_menu_bar=hide_menu_bar,
        hide_status_bar=hide_status_bar,
        fullscreen=fullscreen,
        read_only=read_only,
        macros=macros,
        stylesheet_path=None,
    )

    pydm.utilities.shortcuts.install_connection_inspector(parent=app.main_window)

    # Qt widgets etc.. must be created after the qapplication
    from siriushlacon.utils.images import CNPEM_INVISIBLE_LOGO_ICON

    app.setWindowIcon(CNPEM_INVISIBLE_LOGO_ICON)
    sys.exit(app.exec_())



#!/usr/bin/env python3
import pkg_resources


def get_abs_path(filename):
    return pkg_resources.resource_filename(__name__, filename)


PCTRL_MAIN = get_abs_path("main.py")
PCTRL_UI = get_abs_path("ui/main.ui")
PCTRL_DET_UI = get_abs_path("ui/procServControl.ui")
IOCS_JSON = get_abs_path("iocs.json")



#!/usr/bin/env python3
import logging

import conscommon.data
import pkg_resources

from siriushlacon.utils import LazyDevices

logger = logging.getLogger()


def get_abs_path(filename):
    return pkg_resources.resource_filename(__name__, filename)


MBTEMP_MAIN = get_abs_path("main.py")
MBTEMP_MAIN_UI = get_abs_path("ui/main.ui")

OVERVIEW_MAIN = get_abs_path("overview.py")
OVERVIEW_MAIN_UI = get_abs_path("ui/overview.ui")

BOEXTRACTION_PIC = get_abs_path("images/TL/BOExtraction.png")
BOINJ_PIC = get_abs_path("images/TL/BOInj.png")

BO_PIC1 = get_abs_path("images/BO/Booster1.png")
BO_PIC2 = get_abs_path("images/BO/Booster2.png")
BO_PIC3 = get_abs_path("images/BO/Booster3.png")
BO_PIC4 = get_abs_path("images/BO/Booster4.png")

CNPEM_LOGO = get_abs_path("images/cnpem-logo.png")
LNLS_LOGO = get_abs_path("images/lnls-logo.png")
SRINJ_PIC = get_abs_path("images/TL/SRInj.png")

SR_PIC1 = get_abs_path("images/SR/SR_pic1.png")
SR_PIC2 = get_abs_path("images/SR/SR_pic2.png")
SR_PIC3 = get_abs_path("images/SR/SR_pic3.png")
SR_PIC4 = get_abs_path("images/SR/SR_pic4.png")
SR_PIC5 = get_abs_path("images/SR/SR_pic5.png")
SR_PIC6 = get_abs_path("images/SR/SR_pic6.png")
SR_PIC7 = get_abs_path("images/SR/SR_pic7.png")
SR_PICS = {
    1: SR_PIC1,
    2: SR_PIC2,
    3: SR_PIC3,
    4: SR_PIC4,
    5: SR_PIC5,
    6: SR_PIC6,
    7: SR_PIC7,
}

PIC_PA = get_abs_path("images/Area/PA.jpg")
PIC_LA = get_abs_path("images/Area/LA.png")

PIC_P7RF = get_abs_path("images/Area/Petra7.png")

lazy_devices = LazyDevices(conscommon.data.getMBTemp)



#!/usr/bin/env python3
import enum

import pkg_resources


def get_abs_path(filename):
    return pkg_resources.resource_filename(__name__, filename)


class Finished(str, enum.Enum):
    ON = "ON"
    OFF = "OFF"
    REC = "REC"


# Images
CHECK_IMG = get_abs_path("images/check.png")
CNPEM_IMG = get_abs_path("images/CNPEM.jpg")
LNLS_IMG = get_abs_path("images/LNLS.png")
PLAY_IMG = get_abs_path("images/play_button.jpg")
STOP_IMG = get_abs_path("images/stop_button.png")
WARNING_IMG = get_abs_path("images/warning.png")

# UI
ADVANCED_WINDOW_UI = get_abs_path("ui/advanced_window.ui")
CONFIRMATION_MESSAGE_UI = get_abs_path("ui/confirmation_message.ui")
MAIN_WINDOW_UI = get_abs_path("ui/main_window.ui")
OK_MESSAGE_UI = get_abs_path("ui/OK_message.ui")
SYSTEM_WINDOW_UI = get_abs_path("ui/system_window.ui")
WARNING_WINDOW_UI = get_abs_path("ui/warning_message.ui")
SIMPLE_WINDOW_UI = get_abs_path("ui/simple_window.ui")

# PY
ADVANCED_WINDOW_PY = get_abs_path("advanced_window.py")
CONFIRMATION_MESSAGE_PY = get_abs_path("confirmation_message.py")
MAIN_WINDOW_PY = get_abs_path("main_window.py")
OK_MESSAGE_PY = get_abs_path("OK_message.py")
SYSTEM_WINDOW_PY = get_abs_path("system_window.py")
WARNING_WINDOW_PY = get_abs_path("warning_message.py")
SIMPLE_WINDOW_PY = get_abs_path("simple_window.py")



#!/usr/bin/env python3
import os
import subprocess

from siriushlacon.agilent4uhv.consts import AGILENT_DEVICE_MAIN

os.environ["PYDM_DEFAULT_PROTOCOL"] = "ca://"

subprocess.Popen(
    "pydm -m DEVICE=asd,PREFIX_C1=c1,PREFIX_C2=c2,PREFIX_C3=c3,PREFIX_C4=c4 --hide-nav-bar "
    + AGILENT_DEVICE_MAIN,
    shell=True,
)



#!/usr/bin/env python3
import sys
import typing as _typing

from pydm import Display
from pydm.widgets.byte import PyDMByteIndicator
from pydm.widgets.checkbox import PyDMCheckbox
from pydm.widgets.label import PyDMLabel
from qtpy.QtWidgets import QDialogButtonBox, QLabel, QWidget

from siriushlacon.utils import close_qt_application
from siriushlacon.utils.command_runner import CommandRunner
from siriushlacon.utils.dialog import BaseDialog as _BaseDialog
from siriushlacon.utils.images import LNLS_PIXMAP
from siriushlacon.vbc.consts import WARNING_WINDOW_UI, Finished
from siriushlacon.vbc.OK_message import OkMessageDialog
from siriushlacon.vbc.scripts import process_recovery


class VBCWarningMessage(Display):
    def __init__(
        self,
        parent=None,
        args=None,
        macros=None,
        prefix: _typing.Optional[str] = None,
    ):
        super(VBCWarningMessage, self).__init__(
            parent=parent, args=args, macros=macros, ui_filename=WARNING_WINDOW_UI
        )
        self.prefix = prefix if prefix else str(sys.argv[5])
        self.Stage_1: PyDMByteIndicator
        self.Stage_2: PyDMByteIndicator
        self.Stage_3: PyDMByteIndicator
        self.Stage_4: PyDMByteIndicator
        self.Stage_5: PyDMByteIndicator

        self.pressure_base: PyDMLabel
        self.pressure_exp: PyDMLabel

        self.Shell_PV_Trigger_OK_MESSAGE: PyDMCheckbox
        self.buttonBox_2: QDialogButtonBox
        self.label_9: QLabel

        # defining macro for PyDMShellCommand (for launching "warning_message.ui")
        self.Stage_1.channel = f"ca://{self.prefix}:ProcessRecovery:Status1"
        self.Stage_2.channel = f"ca://{self.prefix}:ProcessRecovery:Status2"
        self.Stage_3.channel = f"ca://{self.prefix}:ProcessRecovery:Status3"
        self.Stage_4.channel = f"ca://{self.prefix}:ProcessRecovery:Status4"
        self.Stage_5.channel = f"ca://{self.prefix}:ProcessRecovery:Status5"
        self.pressure_base.channel = f"ca://{self.prefix}:BBB:TorrBaseMsg"
        self.pressure_exp.channel = f"ca://{self.prefix}:BBB:TorrExpMsg"

        self.label_9.setPixmap(LNLS_PIXMAP)
        self.label_9.setFixedSize(LNLS_PIXMAP.size())

        self.ProcessRecoveryCommand = CommandRunner(
            command=lambda: process_recovery(self.prefix), name="ProcessRecovery"
        )

        self.Shell_PV_Trigger_OK_MESSAGE.setVisible(False)
        self.Shell_PV_Trigger_OK_MESSAGE.toggled.connect(self._display_ok_message)
        self.Shell_PV_Trigger_OK_MESSAGE.toggled.connect(self._close)

        self.buttonBox_2.accepted.connect(
            lambda *_: self.ProcessRecoveryCommand.execute_command()
        )
        self.buttonBox_2.rejected.connect(self._close)

    def _display_ok_message(self, *_):
        dialog = OkMessageDialog(
            parent=self, finished=Finished.REC, prefix=self.prefix, macros=self.macros()
        )
        dialog.show()

    def _close(self):
        if self.parent() and isinstance(self.parent(), QWidget):
            self.parent().close()
        else:
            close_qt_application()


class VBCWarningMessageDialog(_BaseDialog):
    def __init__(
        self,
        parent: _typing.Optional[QWidget],
        macros: _typing.Optional[_typing.Dict[str, str]],
        prefix: str,
    ) -> None:
        super().__init__(parent=parent, window_title="VBC - Warning Message")
        display = VBCWarningMessage(
            parent=self, prefix=prefix, macros=macros if macros else {}
        )
        self.set_display(display)



#!/usr/bin/env python3
import conscommon.data
import pkg_resources

from siriushlacon.utils import LazyDevices


def get_abs_path(filename):
    return pkg_resources.resource_filename(__name__, filename)


AGILENT_OVERVIEW = get_abs_path("overview.py")

AGILENT_MAIN = get_abs_path("main.py")
AGILENT_MAIN_UI = get_abs_path("ui/main.ui")

AGILENT_DEVICE_MAIN = get_abs_path("device_main.py")
AGILENT_DEVICE_MAIN_UI = get_abs_path("ui/device_main.ui")

AGILENT_DEVICE = get_abs_path("device.py")
AGILENT_DEVICE_UI = get_abs_path("ui/device.ui")

AGILENT_CHANNEL = get_abs_path("channel.py")
AGILENT_CHANNEL_UI = get_abs_path("ui/channel.ui")

AGILENT_EXTENDED = get_abs_path("extended.py")


lazy_devices = LazyDevices(conscommon.data.getAgilent)



#!/usr/bin/env python3

from conscommon.data_model.mks import MKS_SENSOR_COLD_CATHODE
from pydm import Display

from siriushlacon.mks937b.consts import PRESSURE_UI
from siriushlacon.utils.consts import IS_LINUX


class Pressure(Display):
    def __init__(self, parent=None, args=None, macros=None):
        super(Pressure, self).__init__(
            parent=parent, args=args, macros=macros, ui_filename=PRESSURE_UI
        )

        if macros:
            if macros.get("A") == MKS_SENSOR_COLD_CATHODE:
                self.alarm2.hide()
                self.lblA2.hide()
                self.cbA2.hide()
                self.biA2.hide()
                self.plblA2.hide()
                if IS_LINUX:
                    self.tpA2.hide()

            if macros.get("B") == MKS_SENSOR_COLD_CATHODE:
                self.lblB2.hide()
                self.alarm4.hide()
                self.lblA2.hide()
                self.cbB2.hide()
                self.biB2.hide()
                self.plblB2.hide()
                if IS_LINUX:
                    self.tpB2.hide()

            if macros.get("C") == MKS_SENSOR_COLD_CATHODE:
                self.lblC2.hide()
                self.alarm6.hide()
                self.lblA2.hide()
                self.cbC2.hide()
                self.biC2.hide()
                self.plblC2.hide()
                if IS_LINUX:
                    self.tpC2.hide()



#!/usr/bin/env python3
import pkg_resources
from setuptools import find_packages, setup

from src.siriushlacon import __author__, __version__


def get_abs_path(relative) -> str:
    return pkg_resources.resource_filename(__name__, relative)


def get_long_description() -> str:
    desc = ""
    with open(get_abs_path("README.md"), "r") as _f:
        desc += _f.read().strip()

    desc += "\n\n"

    with open(get_abs_path("CHANGES.md"), "r") as _f:
        desc += _f.read().strip()

    return desc


long_description = get_long_description()

with open(get_abs_path("requirements.txt"), "r") as _f:
    requirements = _f.read().strip().split("\n")

setup(
    name="siriushlacon",
    author=__author__,
    classifiers=[
        "Intended Audience :: Science/Research",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python",
        "Topic :: Scientific/Engineering",
    ],
    description="Client Applications for Sirius",
    download_url="https://github.com/lnls-sirius/pydm-opi",
    license="GNU GPLv3",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/lnls-sirius/pydm-opi/",
    project_urls={
        "Changelog": "https://github.com/lnls-sirius/pydm-opi/",
    },
    version=__version__,
    install_requires=requirements,
    include_package_data=True,
    packages=find_packages(
        where="src",
        include=[
            "siriushlacon*",
        ],
    ),
    package_dir={"": "src"},
    python_requires=">=3.6",
    scripts=[
        "scripts/sirius-hla-as-ap-bbb-monitor.py",
        "scripts/sirius-hla-as-ap-conlauncher.py",
        "scripts/sirius-hla-as-ap-countingpru.py",
        "scripts/sirius-hla-as-ap-mbtemp.py",
        "scripts/sirius-hla-as-ap-pctrl.py",
        "scripts/sirius-hla-as-ps-regatron-individual.py",
        "scripts/sirius-hla-as-ps-regatron.py",
        "scripts/sirius-hla-as-pu-spixconv.py",
        "scripts/sirius-hla-as-va-agilent4uhv-device.py",
        "scripts/sirius-hla-as-va-agilent4uhv.py",
        "scripts/sirius-hla-as-va-mks937b.py",
        "scripts/sirius-hla-as-va-vbc.py",
        "scripts/sirius-hla-bo-va-agilent4uhv-overview.py",
        "scripts/sirius-hla-bo-va-mks937b-overview.py",
        "scripts/sirius-hla-si-va-agilent4uhv-overview.py",
        "scripts/sirius-hla-si-va-mks937b-overview.py",
        "scripts/sirius-hla-tb-va-agilent4uhv-overview.py",
        "scripts/sirius-hla-tb-va-mks937b-overview.py",
        "scripts/sirius-hla-ts-va-agilent4uhv-overview.py",
        "scripts/sirius-hla-ts-va-mks937b-overview.py",
    ],
    zip_safe=False,
)



#!/usr/bin/env python3

from pydm import Display

from siriushlacon.utils.images import CNPEM_PIXMAP, LNLS_PIXMAP
from siriushlacon.vbc.confirmation_message import ConfirmationMessageDialog
from siriushlacon.vbc.consts import ADVANCED_WINDOW_UI


class DeviceMenu(Display):
    def __init__(self, parent=None, args=None, macros=None):
        super(DeviceMenu, self).__init__(
            parent=parent, args=args, macros=macros, ui_filename=ADVANCED_WINDOW_UI
        )
        self.lnlsLabel.setPixmap(LNLS_PIXMAP)
        self.lnlsLabel.setFixedSize(LNLS_PIXMAP.size())

        self.cnpemLabel.setPixmap(CNPEM_PIXMAP)
        self.cnpemLabel.setFixedSize(CNPEM_PIXMAP.size())

        self.prefix = macros["IOC"]
        self.PyDMCheckbox_Relay1.clicked.connect(lambda *_: self._relay_popup(1))
        self.PyDMCheckbox_Relay2.clicked.connect(lambda *_: self._relay_popup(2))
        self.PyDMCheckbox_Relay3.clicked.connect(lambda *_: self._relay_popup(3))
        self.PyDMCheckbox_Relay4.clicked.connect(lambda *_: self._relay_popup(4))
        self.PyDMCheckbox_Relay5.clicked.connect(lambda *_: self._relay_popup(5))

    def _relay_popup(self, number: int, *_):
        dialog = ConfirmationMessageDialog(
            parent=self, macros=self.macros(), relay_number=number, prefix=self.prefix
        )
        dialog.show()



import json

from pydm import Display
from qtpy.QtGui import QPixmap

from siriushlacon import __version__
from siriushlacon.agilent4uhv.consts import AGILENT_MAIN, AGILENT_OVERVIEW
from siriushlacon.agilent4uhv.extended import MainWindow
from siriushlacon.beaglebones.consts import BEAGLEBONES_MAIN
from siriushlacon.countingpru.consts import GAMMA_COUNTING_MAIN
from siriushlacon.epicstel.consts import EPICSTEL_MAIN
from siriushlacon.launcher.consts import LAUNCH_WINDOW_UI, PCTRL_MAIN
from siriushlacon.mbtemp.consts import MBTEMP_MAIN
from siriushlacon.mks937b.consts import MKS_MAIN, MKS_OVERVIEW
from siriushlacon.regatron.consts import REGATRON_MAIN
from siriushlacon.spixconv.consts import SPIXCONV_MAIN
from siriushlacon.utils.consts import (
    BO,
    CNPEM_INVISIBLE_IMG,
    LNLS_INVISIBLE_IMG,
    SI,
    TB,
    TS,
)
from siriushlacon.vbc.consts import MAIN_WINDOW_PY as VBC_MAIN_WINDOW_PY


class Launcher(Display):
    """
    Vacuum main application interface that gives access to all devices.
    """

    def __init__(self, parent=None, args=None, macros=None, **kwargs):
        super(Launcher, self).__init__(parent=parent, args=args, macros=macros)

        self.lblVersion.setText("v" + __version__)
        self.btnVbc.filenames = [VBC_MAIN_WINDOW_PY]
        self.btnVbc.openInNewWindow = True

        self.btnAgilentExtended.passwordProtected = True
        self.btnAgilentExtended.password = "VACS"
        self.btnAgilentExtended.clicked.connect(self.launchAgilentExtended)

        self.btnAgilent.filenames = [AGILENT_MAIN]
        self.btnAgilent.openInNewWindow = True
        self.btnAgilentSROverview.filenames = [AGILENT_OVERVIEW]
        self.btnAgilentSROverview.macros = json.dumps(
            {
                "device": "UHV",
                "TYPE": SI,
                "TITLE": "ION Pump Agilent 4UHV - SI",
                "FORMAT": "EXP",
            }
        )
        self.btnAgilentSROverview.openInNewWindow = True

        self.btnAgilentBOOverview.filenames = [AGILENT_OVERVIEW]
        self.btnAgilentBOOverview.macros = json.dumps(
            {
                "device": "UHV",
                "TYPE": BO,
                "TITLE": "ION Pump Agilent 4UHV - BO",
                "FORMAT": "EXP",
            }
        )
        self.btnAgilentBOOverview.openInNewWindow = True

        self.btnAgilentTBOverview.filenames = [AGILENT_OVERVIEW]
        self.btnAgilentTBOverview.macros = json.dumps(
            {
                "device": "UHV",
                "TYPE": TB,
                "TITLE": "ION Pump Agilent 4UHV - TB",
                "FORMAT": "EXP",
            }
        )
        self.btnAgilentTBOverview.openInNewWindow = True

        self.btnAgilentTSOverview.filenames = [AGILENT_OVERVIEW]
        self.btnAgilentTSOverview.macros = json.dumps(
            {
                "device": "UHV",
                "TYPE": TS,
                "TITLE": "ION Pump Agilent 4UHV - TS",
                "FORMAT": "EXP",
            }
        )
        self.btnAgilentTSOverview.openInNewWindow = True

        self.btnMks.filenames = [MKS_MAIN]
        self.btnMks.openInNewWindow = True

        self.btnMksSROverview.filenames = [MKS_OVERVIEW]
        self.btnMksSROverview.macros = json.dumps(
            {"device": "MKS", "TYPE": SI, "TITLE": "MKS 937b - SI"}
        )
        self.btnMksSROverview.openInNewWindow = True

        self.btnMksTBOverview.filenames = [MKS_OVERVIEW]
        self.btnMksTBOverview.macros = json.dumps(
            {"device": "MKS", "TYPE": TB, "TITLE": "MKS 937b - TB"}
        )
        self.btnMksTBOverview.openInNewWindow = True

        self.btnMksTSOverview.filenames = [MKS_OVERVIEW]
        self.btnMksTSOverview.macros = json.dumps(
            {"device": "MKS", "TYPE": TS, "TITLE": "MKS 937b - TS"}
        )
        self.btnMksTSOverview.openInNewWindow = True

        self.btnMksBOOverview.filenames = [MKS_OVERVIEW]
        self.btnMksBOOverview.macros = json.dumps(
            {"device": "MKS", "TYPE": BO, "TITLE": "MKS 937b - BO"}
        )
        self.btnMksBOOverview.openInNewWindow = True

        self.btnMBTemp.filenames = [MBTEMP_MAIN]
        self.btnMBTemp.openInNewWindow = True

        self.btnProcServCtrl.filenames = [PCTRL_MAIN]
        self.btnProcServCtrl.openInNewWindow = True

        self.btnRegatron.filenames = [REGATRON_MAIN]
        self.btnRegatron.openInNewWindow = True

        self.btnEpp.filenames = [SPIXCONV_MAIN]
        self.btnEpp.openInNewWindow = True

        self.btnGamma.filenames = [GAMMA_COUNTING_MAIN]
        self.btnGamma.openInNewWindow = True

        self.btnBBB.filenames = [BEAGLEBONES_MAIN]
        self.btnBBB.openInNewWindow = True

        self.btnEPICSTel.filenames = [EPICSTEL_MAIN]
        self.btnEPICSTel.openInNewWindow = True

        self.btnExit.clicked.connect(self.exitApp)

        self.label_cnpem.setPixmap(QPixmap(CNPEM_INVISIBLE_IMG))
        self.label_lnls.setPixmap(QPixmap(LNLS_INVISIBLE_IMG))

    def launchAgilentExtended(self):
        if self.btnAgilentExtended.validate_password():
            # @fixme: We need a better way to launch this
            self.window = MainWindow()
            self.window.show()

    def exitApp(self):
        exit()

    def ui_filename(self):
        return LAUNCH_WINDOW_UI

    def ui_filepath(self):
        return LAUNCH_WINDOW_UI



#!/usr/bin/env python3
from os import path

from pydm.tools import ExternalTool
from pydm.utilities.iconfont import IconFont
from qtpy.QtCore import QUrl
from qtpy.QtGui import QDesktopServices

from siriushlacon.tools.consts import BBB_URL, MGMT_URL, VIEWER_URL


def get_abs_path(relative):
    """
    relative = relative path with base at python/
    """
    return path.join(path.dirname(path.realpath(__file__)), relative)


class ArchiverApplianceTool(ExternalTool):
    def __init__(self):
        icon = IconFont().icon("archive")
        name = "Archiver Appliance"
        group = "CON"
        use_with_widgets = False
        ExternalTool.__init__(
            self, icon=icon, name=name, group=group, use_with_widgets=use_with_widgets
        )

    def call(self, channels, sender):
        QDesktopServices.openUrl(QUrl(MGMT_URL))

    def to_json(self):
        return ""

    def from_json(self, content):
        print("Received from_json: ", content)

    def get_info(self):
        ret = ExternalTool.get_info(self)
        ret.update({"file": __file__})
        return ret


class ArchiverViwerTool(ExternalTool):
    def __init__(self):
        icon = IconFont().icon("desktop")
        name = "Archiver Viwer"
        group = "CON"
        use_with_widgets = False
        ExternalTool.__init__(
            self, icon=icon, name=name, group=group, use_with_widgets=use_with_widgets
        )

    def call(self, channels, sender):
        QDesktopServices.openUrl(QUrl(VIEWER_URL))

    def to_json(self):
        return ""

    def from_json(self, content):
        print("Received from_json: ", content)

    def get_info(self):
        ret = ExternalTool.get_info(self)
        ret.update({"file": __file__})
        return ret


class BeagleboneTool(ExternalTool):
    def __init__(self):
        icon = IconFont().icon("paw")
        name = "Beaglebone Daemon"
        group = "CON"
        use_with_widgets = False
        ExternalTool.__init__(
            self, icon=icon, name=name, group=group, use_with_widgets=use_with_widgets
        )

    def call(self, channels, sender):
        QDesktopServices.openUrl(QUrl(BBB_URL))

    def to_json(self):
        return ""

    def from_json(self, content):
        print("Received from_json: ", content)

    def get_info(self):
        ret = ExternalTool.get_info(self)
        ret.update({"file": __file__})
        return ret



import logging
import time

from siriushlacon.vbc.epics import ACP, BBB, ProcessOn, ProcessRecovery, Turbovac

_logger = logging.getLogger(__name__)


class Initialization:
    def __init__(self, prefix: str):
        if not prefix:
            raise ValueError(f"parameter prefix cannot be empty {prefix}")
        self.prefix = prefix

        self._tick = 0.5

        # Create TURBOVAC PVs
        self.turbovac = Turbovac(prefix=prefix)

        # Create beaglebone PVs
        self.bbb = BBB(prefix=prefix)

        # Create ACP PVs
        self.acp = ACP(prefix=prefix)

        # Create process recovery status PVs
        self.process_recovery = ProcessRecovery(prefix=prefix)

        # Create process ON status PVs
        self.process_on = ProcessOn(prefix=prefix)

    def run(self):
        """if pressure is between 0.05 and 1*10**-8, trigger "process_recovery" script"""
        if (self.bbb.pressure_pv.value < 0.05) & (
            self.bbb.pressure_pv.value > 10 ** -8
        ):
            self._stage_1()
            self._stage_2()
            self._stage_3()
            self._stage_4()
            self._stage_5()

    def _stage_1(self):
        _logger.info("stage1")
        self.process_recovery.status1_pv.value = 1

        # turn ACP15 pump ON and wait 30 s
        self.acp.on_off_pv.value = 1

        # set ACP15 speed to maximum (6000 rpm)
        self.acp.speed_rpm.value = 6000

        # wait until pump receives command to turn on
        while self.acp.on_off_pv.value == 0:
            time.sleep(self._tick)

        time.sleep(30)

    def _stage_2(self):
        _logger.info("stage2")
        # open pre-vacuum valve
        self.bbb.pre_vacuum_valve_sw_pv.value = 1

        # wait gate valve receives command to open
        while self.bbb.pre_vacuum_valve_sw_pv.value == 0:
            time.sleep(self._tick)

        # update status
        self.process_recovery.status2_pv.value = 1

    def _stage_3(self):
        _logger.info("stage3")
        # turn TURBOVAC pump ON
        self.turbovac.pzd1_sp_tevl_pv.value = 1
        self.turbovac.pzd1_sp_zrvl_pv.value = 1

        # wait until pump receives command to turn on
        while (self.turbovac.pzd1_sp_zrvl_pv.value == 0) & (
            self.turbovac.pzd1_sp_tevl_pv.value == 0
        ):
            time.sleep(self._tick)

        self.process_recovery.status3_pv.value = 1

    def _stage_4(self):
        _logger.info(f"stage4, wait until {self.turbovac.pzd2_rb_pv}<1200")
        # wait TURBOVAC pump reaches 1200 Hz
        self.turbovac.pzd2_sp_pv.value = 1200
        self.turbovac.pzd1_sp_sxvl_pv.value = 1

        while self.turbovac.pzd2_rb_pv.value < 1200:
            time.sleep(self._tick)

        self.turbovac.pzd1_sp_sxvl_pv.value = 0

        self.process_recovery.status4_pv.value = 1

    def _stage_5(self):
        _logger.info("stage5")
        # open gate valve (VAT)
        self.bbb.gate_valve_sw_pv.value = 1

        # read gate valve (VAT) status to check if it is really open
        loop = True
        while loop:
            Lo = self.bbb.valve_open_pv.value
            Lg = self.bbb.valve_closed_pv.value
            if Lo & (not Lg):
                loop = False
            time.sleep(self._tick)

        self.process_recovery.status5_pv.value = 1
        self.process_on.activate_all_status()


def initialization(prefix: str):
    """
    this script runs after every boot of BeagleBone Black. It checks whether the
    pressure is lower or higher than 0.05 Torr. In case it's lower than 0.05, then
    run the "process_recovery.py" script.
    """
    init = Initialization(prefix=prefix)
    init.run()
